/*
 *  rommon Simple Monitor
 * 
 *  Copyright (C) 1999-2004 by Dep. of Computer Science and Engineering
 *                   Tomakomai National College of Technology, JAPAN
 *  Copyright (C) 2006-2015 by GJ Business Division
 *                   RICOH COMPANY,LTD. JAPAN
 *  Copyright (C) 2016-2019 by TOPPERS PROJECT Educational Working Group.
 *
 *  L쌠҂́Cȉ (1)`(4) ̏CFree Software Foundation 
 *  ɂČ\Ă GNU General Public License  Version 2 ɋL
 *  qĂ𖞂ꍇɌC{\tgEFAi{\tgEFA
 *  ς̂܂ށDȉjgpEEρEĔzziȉC
 *  pƌĂԁj邱Ƃ𖳏ŋD
 *  (1) {\tgEFA\[XR[ȟ`ŗpꍇɂ́CL̒
 *      \C̗pщL̖ۏ؋K肪Ĉ܂܂̌`Ń\[
 *      XR[hɊ܂܂Ă邱ƁD
 *  (2) {\tgEFACCu`ȂǁC̃\tgEFAJɎg
 *      pł`ōĔzzꍇɂ́CĔzzɔhLgip
 *      ҃}jAȂǁjɁCL̒쌠\C̗pщL
 *      ̖ۏ؋Kfڂ邱ƁD
 *  (3) {\tgEFAC@ɑgݍނȂǁC̃\tgEFAJɎg
 *      płȂ`ōĔzzꍇɂ́Ĉꂩ̏𖞂
 *      ƁD
 *    (a) ĔzzɔhLgip҃}jAȂǁjɁCL̒
 *        쌠\C̗pщL̖ۏ؋Kfڂ邱ƁD
 *    (b) Ĕzž`ԂCʂɒ߂@ɂāCTOPPERSvWFNg
 *        񍐂邱ƁD
 *  (4) {\tgEFA̗pɂ蒼ړI܂͊ԐړIɐ邢Ȃ鑹
 *      QCL쌠҂TOPPERSvWFNgƐӂ邱ƁD
 *
 *  {\tgEFÁCۏ؂Œ񋟂Ă̂łDL쌠҂
 *  TOPPERSvWFNǵC{\tgEFAɊւāC̓Kp\
 *  ܂߂āCȂۏ؂sȂD܂C{\tgEFA̗pɂ蒼
 *  ړI܂͊ԐړIɐȂ鑹QɊւĂC̐ӔC𕉂ȂD
 * 
 *  @(#) $Id: $
 */

#include "sys_defs.h"
#include <sil.h>
#include "encoding.h"
#include <string.h>

/*
 *  SIL֐̃}N`
 */
#define sil_orw_mem(a, b)		sil_wrw_mem((a), sil_rew_mem(a) | (b))
#define sil_andw_mem(a, b)		sil_wrw_mem((a), sil_rew_mem(a) & ~(b))
#define sil_orb_mem(a, b)		sil_wrb_mem((a), sil_reb_mem(a) | (b))
#define sil_andb_mem(a, b)		sil_wrb_mem((a), sil_reb_mem(a) & ~(b))

#define mtime_low_value()       sil_rew_mem((uint32_t *)(TADR_CLIC_BASE+TOFF_CLINT_MTIME))
#define mtime_high_value()      sil_rew_mem((uint32_t *)(TADR_CLIC_BASE+TOFF_CLINT_MTIME+4))

#define DIV		4			/* default DIV value */
#define TRIM    16			/* default TRIM value */
#define F_R     1			/* DIVR to divide-by-2 to get 8MHz frequency (legal values of f_R are 6-12 MHz) */
#define F_F     31			/* DIVF to get 512Mhz frequncy There is an implied multiply-by-2, 16Mhz. So need to write 32-1 */
							/* (legal values of f_F are 384-768 MHz) */
#define F_Q     1			/* DIVQ to divide-by-2 to get 256 MHz frequency (legal values of f_Q are 50-400Mhz) */

uint32_t cpu_frequency = 0;
function_ptr_t plic_interrupt_handlers[TMAX_INTNO];


static unsigned long __attribute__((noinline)) measure_cpu_freq(uint32_t n)
{
	unsigned long start_mtime, delta_mtime;
	unsigned long start_mcycle, delta_mcycle;
	unsigned long tmp = mtime_low_value();

	do{
		start_mtime = mtime_low_value();
	}while(start_mtime == tmp);

	start_mcycle = read_csr(mcycle);
	do{
		delta_mtime = mtime_low_value() - start_mtime;
	}while(delta_mtime < n);

	delta_mcycle = read_csr(mcycle) - start_mcycle;

	return (delta_mcycle / delta_mtime) * TIMER_FREQ
	     + ((delta_mcycle % delta_mtime) * TIMER_FREQ) / delta_mtime;
}


/*
 *  INIT PLIC
 */
void plic_init(void)
{
	unsigned long hart_id = read_csr(mhartid);
	uint32_t i, off, len;

	/*
	 *  Disable all interrupts
	 */
	off = TOFF_PLIC_ENABLE + (hart_id * NUM_PLIC_ENABLE);
	len = (TMAX_INTNO + 8) / 8;
	for(i = 0 ; i < len ; i += 4){
		sil_wrw_mem((uint32_t *)(TADR_PLIC_BASE + off + i), 0);
	}

	/*
	 *  Set all priorities to 0 (equal priority -- don't assume that these are reset).
	 */
	len = (TMAX_INTNO + 1) * sizeof(uint32_t);
	for(i = 0 ; i < len ; i++){
		sil_wrb_mem((uint8_t *)(TADR_PLIC_BASE+TOFF_PLIC_PRIORITY+i), 0);
	}

	/*
	 *  Set the threshold to 0.
	 */
	off = (TOFF_PLIC_THRESHOLD + (hart_id * NUM_PLIC_THRESHOLD));
	sil_wrw_mem((uint32_t *)(TADR_PLIC_BASE+off), 0);

	for(i = 0; i < TMAX_INTNO ; i++){
		plic_interrupt_handlers[i] = NULL;
	}
}

/*
 *  SETUP PRIORITY THRESHOLD
 */
void plic_set_threshold(unsigned long threshold)
{
	unsigned long hart_id = read_csr(mhartid);
	uint32_t off = TOFF_PLIC_THRESHOLD + (hart_id * NUM_PLIC_THRESHOLD);

	sil_wrw_mem((uint32_t *)(TADR_PLIC_BASE+off), threshold);
}

/*
 *  ENABLE PLIC INTERRUPT
 */
void plic_enable_interrupt(unsigned int source)
{
	unsigned long hart_id = read_csr(mhartid);
	uint32_t off = TOFF_PLIC_ENABLE + (hart_id * NUM_PLIC_ENABLE) + (source >> 3);

	sil_orb_mem((uint8_t *)(TADR_PLIC_BASE+off), (1 << (source & 7)));
}

/*
 *  DISABLE PLIC INTERRUPT
 */
void plic_disable_interrupt(unsigned int source)
{
	unsigned long hart_id = read_csr(mhartid);
	uint32_t off = TOFF_PLIC_ENABLE + (hart_id * NUM_PLIC_ENABLE) + (source >> 3);

	sil_andb_mem((uint8_t *)(TADR_PLIC_BASE+off), (1 << (source & 7)));
}

/*
 *  SETUP PLIC INTERRUPT PRIORITY
 */
void plic_set_priority(unsigned int source, unsigned int priority)
{
	uint32_t off;
	if(priority < TMAX_PRIORITY && source < TMAX_INTNO) {
		off = TOFF_PLIC_PRIORITY + (source * sizeof(uint32_t));
		sil_wrb_mem((uint8_t *)(TADR_PLIC_BASE+off), priority);
	}
}

/*
 *  GET CLINT TIMER VALUE
 */
unsigned long long get_timer_value(void)
{
	while (1) {
		uint32_t hi = mtime_high_value();
		uint32_t lo = mtime_low_value();
		if(hi == mtime_high_value())
			return ((uint64_t)hi << 32) | lo;
	}
}


/*
 *  PLIC Interrupt Handler
 */
void m_external_interrupt_handler(void)
{
	unsigned long hart_id = read_csr(mhartid);
	uint32_t int_num;
	int_num = sil_rew_mem((uint32_t *)(TADR_PLIC_BASE+TOFF_PLIC_CLAIM+(hart_id * NUM_PLIC_THRESHOLD)));

	if(int_num >= 1 && int_num < TMAX_INTNO){
		if(plic_interrupt_handlers[int_num] != NULL)
			plic_interrupt_handlers[int_num]();
		else
			sys_printf("m_external_interrupt_handler none handler(%d) !\n", int_num);
	}
	else{
		sys_printf("m_external_interrupt_handler Illegal global interrupt(%d) !\n", int_num);
	}
	sil_wrw_mem((uint32_t *)(TADR_PLIC_BASE+TOFF_PLIC_CLAIM+(hart_id * NUM_PLIC_THRESHOLD)), int_num);
}


long handle_trap(long mcause, long epc)
{
    /*
	 *  Handle External Machine-Level plic interrupt
	 */
	if((mcause & MCAUSE_INT) && ((mcause & MCAUSE_CAUSE) == IRQ_M_EXT)){
		m_external_interrupt_handler();
	}
	/*
	 *  Handle Machine-Level exception
	 */
	else if((mcause & MCAUSE_INT) == 0){
		sys_printf("handle_trap Exception occurs(%d) !\n", (mcause & MCAUSE_CAUSE));
	}
	else{
		sys_printf("handle_trap Illegal machine interrupt(%d) !\n", (mcause & MCAUSE_CAUSE));
	}
	return epc;
}

void BeforeInit(void)
{
#ifdef DEFAULT_CLOCK
	/*
	 *  Turn off the LFROSC
	 */
	sil_andw_mem((uint32_t *)(TADR_AON_BASE+TOFF_AON_LFROSC), LFROSC_OSC_EN);

	/*
	 *  Setup the HFROSC using default running value.
	 */
	sil_wrw_mem((uint32_t *)(TADR_PRCI_BASE+PRCI_HFROSCCFG), ((DIV << 0) | (TRIM << 16) | HFROSCCFG_OSC_EN));
	while((sil_rew_mem((uint32_t *)(TADR_PRCI_BASE+PRCI_HFROSCCFG)) & HFROSCCFG_OSC_RDY) == 0);
	sil_andw_mem((uint32_t *)(TADR_PRCI_BASE+PRCI_PLLCFG), PLLCFG_PLL_SEL);

	/* In case we are executing from QSPI,
	 * (which is quite likely) we need to
	 * set the QSPI clock divider appropriately
	 * before boosting the clock frequency. */
	/*
	 *  Div = f_sck/2
	 */
	sil_wrw_mem((uint32_t *)(TADR_SPI0_BASE+TOFF_SPI_SCKDIV), 8);

	/*
	 *  Set our Final output divide to divide-by-1.
	 */
	sil_wrw_mem((uint32_t *)(TADR_PRCI_BASE+PRCI_PLLDIV), (PLL_FINAL_DIV_BY_1(1) | PLL_FINAL_DIV(0)));

	/*
	 *  Setup PLL with 16MHz source clock.
	 */
	sil_wrw_mem((uint32_t *)(TADR_PRCI_BASE+PRCI_PLLCFG), (PLLCFG_PLL_BYPASS | (F_R << 0) | (F_F << 4) | (F_Q << 10)));

	/*
	 *  Not Bypass the PLL
	 */
	sil_andw_mem((uint32_t *)(TADR_PRCI_BASE+PRCI_PLLCFG), PLLCFG_PLL_BYPASS);

	/*
	 *  Wait for PLL Lock 100 us
	 */
	sil_dly_nse(100*1000);

	/*
	 *  Check PLL Lock
	 */
	while((sil_rew_mem((uint32_t *)(TADR_PRCI_BASE+PRCI_PLLCFG)) & PLLCFG_PLL_LOCK) == 0) ;

	/*
	 *  Select PLL clock source
	 */
	sil_orw_mem((uint32_t *)(TADR_PRCI_BASE+PRCI_PLLCFG), PLLCFG_SEL_PLL);
#else
	/*
	 *  Setup the HFROSC using default running value.
	 */
	sil_wrw_mem((uint32_t *)(TADR_PRCI_BASE+PRCI_HFROSCCFG), ((DIV << 0) | (TRIM << 16) | HFROSCCFG_OSC_EN));

	/*
	 *  PLL running original clock(16MHz).
	 */
	sil_wrw_mem((uint32_t *)(TADR_PRCI_BASE+PRCI_PLLCFG), (PLLCFG_PLL_REFSEL | PLLCFG_PLL_BYPASS));
	sil_orw_mem((uint32_t *)(TADR_PRCI_BASE+PRCI_PLLCFG), PLLCFG_SEL_PLL);

	/*
	 * Turn off HFROSC (power save mode)
	 */
	sil_andw_mem((uint32_t *)(TADR_PRCI_BASE+PRCI_HFROSCCFG), HFROSCCFG_OSC_EN);
#endif

#ifndef ENTRY_S
	write_csr(mtvec, &trap_entry);
#endif
	if(read_csr(misa) & (1 << ('F' - 'A'))) { // if F extension is present
		write_csr(mstatus, MSTATUS_FS); // allow FPU instructions without trapping
		write_csr(fcsr, 0); // initialize rounding mode, undefined at reset
	}

	/*
	 *  CALCULATE the CPU frequency
	 */
	measure_cpu_freq(1);
	cpu_frequency = measure_cpu_freq(10);
}

