
#include "sys_defs.h"
#include <stdio.h>
#include "riscv_encoding.h"
#include "sil.h"

/*
 *  SIL֐̃}N`
 */
#define sil_orw_mem(a, b)		sil_wrw_mem((a), sil_rew_mem(a) | (b))
#define sil_andw_mem(a, b)		sil_wrw_mem((a), sil_rew_mem(a) & ~(b))
#define sil_modw_mem(a, b, c)	sil_wrw_mem((a), (sil_rew_mem(a) & (~b)) | (c))
#define sil_orb_mem(a, b)		sil_wrb_mem((a), sil_reb_mem(a) | (b))
#define sil_andb_mem(a, b)		sil_wrb_mem((a), sil_reb_mem(a) & ~(b))
#define sil_modb_mem(a, b, c)	sil_wrb_mem((a), (sil_reb_mem(a) & (~b)) | (c))

#define mtime_low_value()       sil_rew_mem((uint32_t *)(TADR_CLIC_BASE+TOFF_CLINT_MTIME))
#define mtime_high_value()      sil_rew_mem((uint32_t *)(TADR_CLIC_BASE+TOFF_CLINT_MTIME+4))


#define ECLICINTCTLBITS         4
#define ECLIC_CFG_NLBITS_MASK   0x1EU
#define ECLIC_CFG_NLBITS_LSB    1U

#define HXTAL_STARTUP_TIMEOUT   (500*1000)	/* 500ms */


/*
 *  VXeNbNI
 */
#define SEL_IRC8M               0x00U
#define SEL_HXTAL               0x04U
#define SEL_PLL                 0x08U

/*
 *  MULTI FACTOR SHIFT
 */
#define PLL_MFSHIFT             18
#define PLL1_MFSHIFT            8
#define PLL2_MFSHIFT            12

/*
 *  VXeNbN\[X`
 */
#define RCU_CKSYSSRC_IRC8M      0x00000000			/* system clock source select IRC8M */
#define RCU_CKSYSSRC_HXTAL      RCU_CFG0_SCS_0		/* system clock source select HXTAL */
#define RCU_CKSYSSRC_PLL        RCU_CFG0_SCS_1		/* system clock source select PLL */

/*
 *  VXeNbN\[XI
 */
#define RCU_SCSS_IRC8M          0x00000000			/* system clock source select IRC8M */
#define RCU_SCSS_HXTAL          RCU_CFG0_SCSS_0		/* system clock source select HXTAL */
#define RCU_SCSS_PLL            RCU_CFG0_SCSS_1		/* system clock source select PLLP */

/*
 *  PLL VXeNbN\[XI
 */
#define RCU_PLLSRC_IRC8M_DIV2   0x00000000U			/* IRC8M/2 clock selected as source clock of PLL */
#define RCU_PLLSRC_HXTAL        RCU_CFG0_PLLSEL		/* HXTAL clock selected as source clock of PLL */

/*
 *  PREDV0 VXeNbN\[XI
 */
#define RCU_PREDV0SRC_HXTAL     0x00000000U			/* HXTAL selected as PREDV0 input source clock */
#define RCU_PREDV0SRC_CKPLL1    RCU_CFG1_PREDV0SEL	/* CK_PLL1 selected as PREDV0 input source clock */


/*
 *  AHBvXP[`
 */
#define RCU_AHB_CKSYS_DIV1      0x00000000			/* AHB prescaler select CK_SYS */
#define RCU_AHB_CKSYS_DIV2      RCU_CFG0_AHBPSC_3	/* AHB prescaler select CK_SYS/2 */
#define RCU_AHB_CKSYS_DIV4      (RCU_CFG0_AHBPSC_0 | RCU_CFG0_AHBPSC_3)	/* AHB prescaler select CK_SYS/4 */
#define RCU_AHB_CKSYS_DIV8      (RCU_CFG0_AHBPSC_1 | RCU_CFG0_AHBPSC_3)	/* AHB prescaler select CK_SYS/8 */
#define RCU_AHB_CKSYS_DIV16     (RCU_CFG0_AHBPSC_0 | RCU_CFG0_AHBPSC_1 | RCU_CFG0_AHBPSC_3)	/* AHB prescaler select CK_SYS/16 */
#define RCU_AHB_CKSYS_DIV64     (RCU_CFG0_AHBPSC_2 | RCU_CFG0_AHBPSC_3)	/* AHB prescaler select CK_SYS/64 */
#define RCU_AHB_CKSYS_DIV128    (RCU_CFG0_AHBPSC_0 | RCU_CFG0_AHBPSC_2 | RCU_CFG0_AHBPSC_3)	/* AHB prescaler select CK_SYS/128 */
#define RCU_AHB_CKSYS_DIV256    (RCU_CFG0_AHBPSC_1 | RCU_CFG0_AHBPSC_2 | RCU_CFG0_AHBPSC_3)	/* AHB prescaler select CK_SYS/256 */
#define RCU_AHB_CKSYS_DIV512    (RCU_CFG0_AHBPSC_0 | RCU_CFG0_AHBPSC_1 | RCU_CFG0_AHBPSC_2 | RCU_CFG0_AHBPSC_3)

/*
 *  APB1vXP[`
 */
#define RCU_APB1_CKAHB_DIV1     0x00000000			/* APB1 prescaler select CK_AHB */
#define RCU_APB1_CKAHB_DIV2     RCU_CFG0_APB1PSC_2	/* APB1 prescaler select CK_AHB/2 */
#define RCU_APB1_CKAHB_DIV4     (RCU_CFG0_APB1PSC_0 | RCU_CFG0_APB1PSC_2)	/* APB1 prescaler select CK_AHB/4 */
#define RCU_APB1_CKAHB_DIV8     (RCU_CFG0_APB1PSC_1 | RCU_CFG0_APB1PSC_2)	/* APB1 prescaler select CK_AHB/8 */
													/* APB1 prescaler select CK_AHB/16 */
#define RCU_APB1_CKAHB_DIV16    (RCU_CFG0_APB1PSC_0 | RCU_CFG0_APB1PSC_1 | RCU_CFG0_APB1PSC_2)

/*
 *  APB2 vXP[`
 */
#define RCU_APB2_CKAHB_DIV1     0x00000000			/* APB2 prescaler select CK_AHB */
#define RCU_APB2_CKAHB_DIV2     RCU_CFG0_APB2PSC_2	/* APB2 prescaler select CK_AHB/2 */
#define RCU_APB2_CKAHB_DIV4     (RCU_CFG0_APB2PSC_0 | RCU_CFG0_APB2PSC_2)	/* APB2 prescaler select CK_AHB/4 */
#define RCU_APB2_CKAHB_DIV8     (RCU_CFG0_APB2PSC_1 | RCU_CFG0_APB2PSC_2)	/* APB2 prescaler select CK_AHB/8 */
													/* APB2 prescaler select CK_AHB/16 */
#define RCU_APB2_CKAHB_DIV16    (RCU_CFG0_APB2PSC_0 | RCU_CFG0_APB2PSC_1 | RCU_CFG0_APB2PSC_2)

/*
 *  PREDV0t@N^[`
 */
#define RCU_PREDV0_DIV1         0x00000000			/* PREDV0 input source clock not divided */
#define RCU_PREDV0_DIV2         RCU_CFG1_PREDV0_0	/* PREDV0 input source clock divided by 2 */
#define RCU_PREDV0_DIV3         RCU_CFG1_PREDV0_1	/* PREDV0 input source clock divided by 3 */
#define RCU_PREDV0_DIV4         (RCU_CFG1_PREDV0_0 | RCU_CFG1_PREDV0_1)	/* PREDV0 input source clock divided by 4 */
#define RCU_PREDV0_DIV5         RCU_CFG1_PREDV0_2	/* PREDV0 input source clock divided by 5 */
#define RCU_PREDV0_DIV6         (RCU_CFG1_PREDV0_0 | RCU_CFG1_PREDV0_2)	/* PREDV0 input source clock divided by 6 */
#define RCU_PREDV0_DIV7         (RCU_CFG1_PREDV0_1 | RCU_CFG1_PREDV0_2)	/* PREDV0 input source clock divided by 7 */
#define RCU_PREDV0_DIV8         (RCU_CFG1_PREDV0_0 | RCU_CFG1_PREDV0_1 | RCU_CFG1_PREDV0_2)
#define RCU_PREDV0_DIV9         RCU_CFG1_PREDV0_3	/* PREDV0 input source clock divided by 9 */
#define RCU_PREDV0_DIV10        (RCU_CFG1_PREDV0_0 | RCU_CFG1_PREDV0_3)	/* PREDV0 input source clock divided by 10 */
#define RCU_PREDV0_DIV11        (RCU_CFG1_PREDV0_1 | RCU_CFG1_PREDV0_3)	/* PREDV0 input source clock divided by 11 */
#define RCU_PREDV0_DIV12        (RCU_CFG1_PREDV0_0 | RCU_CFG1_PREDV0_1 | RCU_CFG1_PREDV0_3)
#define RCU_PREDV0_DIV13        (RCU_CFG1_PREDV0_2 | RCU_CFG1_PREDV0_3)	/* PREDV0 input source clock divided by 13 */
#define RCU_PREDV0_DIV14        (RCU_CFG1_PREDV0_0 | RCU_CFG1_PREDV0_2 | RCU_CFG1_PREDV0_3)
#define RCU_PREDV0_DIV15        (RCU_CFG1_PREDV0_1 | RCU_CFG1_PREDV0_2 | RCU_CFG1_PREDV0_3)
#define RCU_PREDV0_DIV16        (RCU_CFG1_PREDV0_0 | RCU_CFG1_PREDV0_1 | RCU_CFG1_PREDV0_2 | RCU_CFG1_PREDV0_3)

/*
 *  PREDV1t@N^[`
 */
#define CFG1_PREDV1(regval)             (BITS(4,7) & ((uint32_t)(regval) << 4))
#define RCU_PREDV1_DIV1         0x00000000			/* PREDV1 input source clock not divided */
#define RCU_PREDV1_DIV2         RCU_CFG1_PREDV1_0	/* PREDV1 input source clock divided by 2 */
#define RCU_PREDV1_DIV3         RCU_CFG1_PREDV1_1	/* PREDV1 input source clock divided by 3 */
#define RCU_PREDV1_DIV4         (RCU_CFG1_PREDV1_0 | RCU_CFG1_PREDV1_1)	/* PREDV1 input source clock divided by 4 */
#define RCU_PREDV1_DIV5         RCU_CFG1_PREDV1_2	/* PREDV1 input source clock divided by 5 */
#define RCU_PREDV1_DIV6         (RCU_CFG1_PREDV1_0 | RCU_CFG1_PREDV1_2)	/* PREDV1 input source clock divided by 6 */
#define RCU_PREDV1_DIV7         (RCU_CFG1_PREDV1_1 | RCU_CFG1_PREDV1_2)	/* PREDV1 input source clock divided by 7 */
#define RCU_PREDV1_DIV8         (RCU_CFG1_PREDV1_0 | RCU_CFG1_PREDV1_1 | RCU_CFG1_PREDV1_2)
#define RCU_PREDV1_DIV9         RCU_CFG1_PREDV0_3	/* PREDV1 input source clock divided by 9 */
#define RCU_PREDV1_DIV10        (RCU_CFG1_PREDV0_0 | RCU_CFG1_PREDV0_3)	/* PREDV1 input source clock divided by 10 */
#define RCU_PREDV1_DIV11        (RCU_CFG1_PREDV0_1 | RCU_CFG1_PREDV0_3)	/* PREDV1 input source clock divided by 11 */
#define RCU_PREDV1_DIV12        (RCU_CFG1_PREDV0_0 | RCU_CFG1_PREDV0_1 | RCU_CFG1_PREDV0_3)
#define RCU_PREDV1_DIV13        (RCU_CFG1_PREDV0_2 | RCU_CFG1_PREDV0_3)	/* PREDV1 input source clock divided by 13 */
#define RCU_PREDV1_DIV14        (RCU_CFG1_PREDV0_0 | RCU_CFG1_PREDV0_2 | RCU_CFG1_PREDV0_3)
#define RCU_PREDV1_DIV15        (RCU_CFG1_PREDV0_1 | RCU_CFG1_PREDV0_1 | RCU_CFG1_PREDV0_3)
#define RCU_PREDV1_DIV16        (RCU_CFG1_PREDV0_0 | RCU_CFG1_PREDV0_1 | RCU_CFG1_PREDV0_2 | RCU_CFG1_PREDV0_3)

/*
 *  PLL clock multiplication factor
 */
#define RCU_PLL_MUL2            (0<<PLL_MFSHIFT)		/* PLL source clock multiply by 2 */
#define RCU_PLL_MUL3            (1<<PLL_MFSHIFT)		/* PLL source clock multiply by 3 */
#define RCU_PLL_MUL4            (2<<PLL_MFSHIFT)		/* PLL source clock multiply by 4 */
#define RCU_PLL_MUL5            (3<<PLL_MFSHIFT)		/* PLL source clock multiply by 5 */
#define RCU_PLL_MUL6            (4<<PLL_MFSHIFT)		/* PLL source clock multiply by 6 */
#define RCU_PLL_MUL7            (5<<PLL_MFSHIFT)		/* PLL source clock multiply by 7 */
#define RCU_PLL_MUL8            (6<<PLL_MFSHIFT)		/* PLL source clock multiply by 8 */
#define RCU_PLL_MUL9            (7<<PLL_MFSHIFT)		/* PLL source clock multiply by 9 */
#define RCU_PLL_MUL10           (8<<PLL_MFSHIFT)		/* PLL source clock multiply by 10 */
#define RCU_PLL_MUL11           (9<<PLL_MFSHIFT)		/* PLL source clock multiply by 11 */
#define RCU_PLL_MUL12           (10<<PLL_MFSHIFT)		/* PLL source clock multiply by 12 */
#define RCU_PLL_MUL13           (11<<PLL_MFSHIFT)		/* PLL source clock multiply by 13 */
#define RCU_PLL_MUL14           (12<<PLL_MFSHIFT)		/* PLL source clock multiply by 14 */
#define RCU_PLL_MUL6_5          (13<<PLL_MFSHIFT)		/* PLL source clock multiply by 6.5 */
#define RCU_PLL_MUL16           (14<<PLL_MFSHIFT)		/* PLL source clock multiply by 16 */
#define RCU_PLL_MUL17           (RCU_CFG0_PLLMF_4 | (0<<PLL_MFSHIFT))	/* PLL source clock multiply by 17 */
#define RCU_PLL_MUL18           (RCU_CFG0_PLLMF_4 | (1<<PLL_MFSHIFT))	/* PLL source clock multiply by 18 */
#define RCU_PLL_MUL19           (RCU_CFG0_PLLMF_4 | (2<<PLL_MFSHIFT))	/* PLL source clock multiply by 19 */
#define RCU_PLL_MUL20           (RCU_CFG0_PLLMF_4 | (3<<PLL_MFSHIFT))	/* PLL source clock multiply by 20 */
#define RCU_PLL_MUL21           (RCU_CFG0_PLLMF_4 | (4<<PLL_MFSHIFT))	/* PLL source clock multiply by 21 */
#define RCU_PLL_MUL22           (RCU_CFG0_PLLMF_4 | (5<<PLL_MFSHIFT))	/* PLL source clock multiply by 22 */
#define RCU_PLL_MUL23           (RCU_CFG0_PLLMF_4 | (6<<PLL_MFSHIFT))	/* PLL source clock multiply by 23 */
#define RCU_PLL_MUL24           (RCU_CFG0_PLLMF_4 | (7<<PLL_MFSHIFT))	/* PLL source clock multiply by 24 */
#define RCU_PLL_MUL25           (RCU_CFG0_PLLMF_4 | (8<<PLL_MFSHIFT))	/* PLL source clock multiply by 25 */
#define RCU_PLL_MUL26           (RCU_CFG0_PLLMF_4 | (9<<PLL_MFSHIFT))	/* PLL source clock multiply by 26 */
#define RCU_PLL_MUL27           (RCU_CFG0_PLLMF_4 | (10<<PLL_MFSHIFT))	/* PLL source clock multiply by 27 */
#define RCU_PLL_MUL28           (RCU_CFG0_PLLMF_4 | (11<<PLL_MFSHIFT))	/* PLL source clock multiply by 28 */
#define RCU_PLL_MUL29           (RCU_CFG0_PLLMF_4 | (12<<PLL_MFSHIFT))	/* PLL source clock multiply by 29 */
#define RCU_PLL_MUL30           (RCU_CFG0_PLLMF_4 | (13<<PLL_MFSHIFT))	/* PLL source clock multiply by 30 */
#define RCU_PLL_MUL31           (RCU_CFG0_PLLMF_4 | (14<<PLL_MFSHIFT))	/* PLL source clock multiply by 31 */
#define RCU_PLL_MUL32           (RCU_CFG0_PLLMF_4 | (15<<PLL_MFSHIFT))	/* PLL source clock multiply by 32 */

/*
 *  PLL1 clock multiplication factor
 */
#define RCU_PLL1_MUL8           (6<<PLL1_MFSHIFT)	/* PLL1 source clock multiply by 8 */
#define RCU_PLL1_MUL9           (7<<PLL1_MFSHIFT)	/* PLL1 source clock multiply by 9 */
#define RCU_PLL1_MUL10          (8<<PLL1_MFSHIFT)	/* PLL1 source clock multiply by 10 */
#define RCU_PLL1_MUL11          (9<<PLL1_MFSHIFT)	/* PLL1 source clock multiply by 11 */
#define RCU_PLL1_MUL12          (10<<PLL1_MFSHIFT)	/* PLL1 source clock multiply by 12 */
#define RCU_PLL1_MUL13          (11<<PLL1_MFSHIFT)	/* PLL1 source clock multiply by 13 */
#define RCU_PLL1_MUL14          (12<<PLL1_MFSHIFT)	/* PLL1 source clock multiply by 14 */
#define RCU_PLL1_MUL15          (13<<PLL1_MFSHIFT)	/* PLL1 source clock multiply by 15 */
#define RCU_PLL1_MUL16          (14<<PLL1_MFSHIFT)	/* PLL1 source clock multiply by 16 */
#define RCU_PLL1_MUL20          (15<<PLL1_MFSHIFT)	/* PLL1 source clock multiply by 20 */

/*
 *  PLL2 clock multiplication factor
 */
#define RCU_PLL2_MUL8           (6<<PLL2_MFSHIFT)	/* PLL2 source clock multiply by 8 */
#define RCU_PLL2_MUL9           (7<<PLL2_MFSHIFT)	/* PLL2 source clock multiply by 9 */
#define RCU_PLL2_MUL10          (8<<PLL2_MFSHIFT)	/* PLL2 source clock multiply by 10 */
#define RCU_PLL2_MUL11          (9<<PLL2_MFSHIFT)	/* PLL2 source clock multiply by 11 */
#define RCU_PLL2_MUL12          (10<<PLL2_MFSHIFT)	/* PLL2 source clock multiply by 12 */
#define RCU_PLL2_MUL13          (11<<PLL2_MFSHIFT)	/* PLL2 source clock multiply by 13 */
#define RCU_PLL2_MUL14          (12<<PLL2_MFSHIFT)	/* PLL2 source clock multiply by 14 */
#define RCU_PLL2_MUL15          (13<<PLL2_MFSHIFT)	/* PLL2 source clock multiply by 15 */
#define RCU_PLL2_MUL16          (14<<PLL2_MFSHIFT)	/* PLL2 source clock multiply by 16 */
#define RCU_PLL2_MUL20          (15<<PLL2_MFSHIFT)	/* PLL2 source clock multiply by 20 */


#define RCU_CTL_CONFIG1     (RCU_CTL_HXTALEN | RCU_CTL_CKMEN | RCU_CTL_PLLEN)
#define RCU_CTL_CONFIG2     (RCU_CTL_PLLEN | RCU_CTL_PLL1EN | RCU_CTL_PLL2EN | RCU_CTL_CKMEN | RCU_CTL_HXTALEN)
#define RCU_CFG0_CONFIG1    (RCU_CFG0_SCS | RCU_CFG0_AHBPSC | RCU_CFG0_APB1PSC | \
							 RCU_CFG0_APB2PSC | RCU_CFG0_ADCPSC | RCU_CFG0_ADCPSC_2 | RCU_CFG0_CKOUT0SEL)
#define RCU_CFG0_CONFIG2    (RCU_CFG0_PLLSEL | RCU_CFG0_PREDV0_LSB | RCU_CFG0_PLLMF | \
							 RCU_CFG0_USBFSPSC | RCU_CFG0_PLLMF_4)



/*
 * exponent of AHB, APB1 and APB2 clock divider
 */
static const uint8_t ahb_exp[16] = {0, 0, 0, 0, 0, 0, 0, 0, 1, 2, 3, 4, 6, 7, 8, 9};
static const uint8_t apb1_exp[8] = {0, 0, 0, 0, 1, 2, 3, 4};
static const uint8_t apb2_exp[8] = {0, 0, 0, 0, 1, 2, 3, 4};

unsigned int   cpu_frequency;

#ifndef TIMER_FREQ
#define TIMER_FREQ ((uint32_t)SYSTEM_CLOCK_PLL_HXTAL/4)
#endif

extern uint32_t disable_mcycle_minstret();

static unsigned long __attribute__((noinline)) measure_cpu_freq(uint32_t n)
{
	unsigned long start_mtime, delta_mtime;
	unsigned long start_mcycle, delta_mcycle;
	unsigned long tmp = mtime_low_value();

	do{
		start_mtime = mtime_low_value();
	}while(start_mtime == tmp);

	start_mcycle = read_csr(mcycle);
	do{
		delta_mtime = mtime_low_value() - start_mtime;
	}while(delta_mtime < n);

	delta_mcycle = read_csr(mcycle) - start_mcycle;

	return (delta_mcycle / delta_mtime) * TIMER_FREQ
	     + ((delta_mcycle % delta_mtime) * TIMER_FREQ) / delta_mtime;
}

/*
 *  INIT ECLIC
 */
void eclic_init(void)
{
	uint32_t base, i;

	/*
	 *  RtBMEXbV WX^
	 */
	eclic_priority_group_set(ECLIC_PRIGROUP_LEVEL0_PRIO4);
	sil_wrb_mem((uint8_t *)(TADR_CLIC_BASE+TOFF_ECLIC_MTH), 0);

	/*
	 *  IP/IE/ATTR/CTRL̏
	 */
	base = TADR_CLIC_BASE+TOFF_ECLIC_INT_IP;
	for(i = 0 ; i < (TMAX_INTNO*4) ; i += 4){
		sil_wrw_mem((uint32_t *)(base+i), 0);
	}
}

/*
 *  ECLIC[hݒ
 */
void eclic_mode_enable(void)
{
  uint32_t mtvec_value = read_csr(mtvec);
  mtvec_value = mtvec_value & 0xFFFFFFC0;
  mtvec_value = mtvec_value | 0x00000003;
  write_csr(mtvec,mtvec_value);
}


/*
 *  DxO[vݒ
 *  parameter1:  priority groupݒ
 */
void eclic_priority_group_set(uint32_t prigroup)
{
	//shift nlbits to correct position
	uint8_t nlbits_shifted = (prigroup << ECLIC_CFG_NLBITS_LSB) & ECLIC_CFG_NLBITS_MASK;
	sil_modb_mem((uint8_t *)(TADR_ECLIC_BASE+TOFF_ECLIC_CFG), ECLIC_CFG_NLBITS_MASK, nlbits_shifted);
}

/*
 *  ECLICݗL
 *  parameter1:  ݃\[X
 *  parameter2:  ݃xݒ(0ɌŒ)
 *  parameter3:  ݗDxݒ(0-15)
 */
void eclic_irq_enable(uint32_t source, uint8_t lvl_abs, uint8_t priority)
{
	uint8_t current_intctrl, lvl;
	uint8_t nlbits = sil_reb_mem((uint8_t *)(TADR_ECLIC_BASE+TOFF_ECLIC_CFG));

	nlbits = (nlbits & ECLIC_CFG_NLBITS_MASK) >> ECLIC_CFG_NLBITS_LSB;

	if (nlbits > ECLICINTCTLBITS)
		nlbits = ECLICINTCTLBITS;

	/*
	 *  ݃xݒ(ݒ0ƂȂ)
	 */
	current_intctrl = sil_reb_mem((uint8_t *)(TADR_ECLIC_BASE+TOFF_ECLIC_INT_CTRL+source*4));
	lvl = lvl_abs << (8-nlbits);
	current_intctrl = current_intctrl << nlbits;
	current_intctrl = current_intctrl >> nlbits;
	current_intctrl |=  lvl;

	/*
	 *  ݗDxݒ
	 */
	priority = priority << (8 - ECLICINTCTLBITS);
	current_intctrl = current_intctrl >> (8-nlbits);
	current_intctrl = current_intctrl << (8-nlbits);
	current_intctrl |= priority;
	sil_wrb_mem((uint8_t *)(TADR_ECLIC_BASE+TOFF_ECLIC_INT_CTRL+source*4), current_intctrl);

	/*
	 *  ݗL
	 */
	sil_wrb_mem((uint8_t *)(TADR_ECLIC_BASE+TOFF_ECLIC_INT_IE+source*4), 1);
}

/*
 *  ECLICݖ
 *  parameter1:  ݃\[X
 */
void eclic_irq_disable(uint32_t source)
{
	sil_wrb_mem((uint8_t *)(TADR_ECLIC_BASE+TOFF_ECLIC_INT_IE+source*4), 0);
}

/*
 *  ECLICݗDx擾
 *  parameter1:  ݃\[X
 *  return:      ݗDx
 */
uint8_t eclic_get_irq_priority(uint32_t source)
{
	uint8_t intctrl, priority;
	uint8_t nlbits = sil_reb_mem((uint8_t *)(TADR_ECLIC_BASE+TOFF_ECLIC_CFG));
	nlbits = (nlbits & ECLIC_CFG_NLBITS_MASK) >> ECLIC_CFG_NLBITS_LSB;
	if (nlbits > ECLICINTCTLBITS) {
		nlbits = ECLICINTCTLBITS;
	}

	intctrl = sil_reb_mem((uint8_t *)(TADR_ECLIC_BASE+TOFF_ECLIC_INT_CTRL+source*4));
	intctrl = intctrl << nlbits;
	priority = intctrl >> (nlbits+(8 - ECLICINTCTLBITS));
	return priority;
}

/*
 *  ELIC Interrupt Handler
 */
void m_external_interrupt_handler(void)
{
	sys_printf("m_external_interrupt_handler Illegal global interrupt !\n");
}

/*
 *  TRAPnh
 */
long handle_trap(long mcause, long epc)
{
    /*
	 *  Handle External Machine-Level plic interrupt
	 */
	if((mcause & MCAUSE_INT) && ((mcause & MCAUSE_CAUSE) == IRQ_M_EXT)){
		m_external_interrupt_handler();
	}
	/*
	 *  Handle Machine-Level exception
	 */
	else if((mcause & MCAUSE_INT) == 0){
		sys_printf("Exception occurs(%d) !\n", (mcause & MCAUSE_CAUSE));
	}
	else{
		sys_printf("Illegal machine interrupt(%d) !\n", (mcause & MCAUSE_CAUSE));
	}
	return epc;
}


/*
 *  VXeNbNݒ
 *  ݒl͈ȉ̒ʂ
 *      System Clock source            = PLL
 *      SYSCLK(Hz)                     = 108000000
 *      AHB Prescaler                  = 1
 *      APB1 Prescaler                 = 2
 *      APB2 Prescaler                 = 1
 *      PREDV0 Prescaler               = 2
 *      PREDV1 Prescaler               = 2
 *      CK_PLL                         = (CK_PREDIV0) * 27 = 108 MHz
 *      HSI Frequency(Hz)              = 16000000
 *      PLL_MUL                        = 27
 *      PLL1_MUL                       = 20
 *      PLL2_MUL                       = 20
 *  return  IȂ0Ԃ
 */
int SystemInit(void)
{
    uint32_t timeout   = 0U;
    uint32_t stab_flag = 0U;

    /* reset the RCC clock configuration to the default reset state */
    /*
	 *  IRC8ML
	 */
	sil_orw_mem((uint32_t *)(TADR_RCU_BASE+TOFF_RCU_CTL), RCU_CTL_IRC8MEN);

    /*
	 *  SCS, AHBPSC, APB1PSC, APB2PSC, ADCPSC, CKOUT0SELZbg
	 */
	sil_andw_mem((uint32_t *)(TADR_RCU_BASE+TOFF_RCU_CFG0), RCU_CFG0_CONFIG1);

    /*
	 *  HXTALEN, CKMEN, PLLENZbg
	 */
	sil_andw_mem((uint32_t *)(TADR_RCU_BASE+TOFF_RCU_CTL), RCU_CTL_CONFIG1);

    /*
	 *  HXTALBPSZbg
	 */
	sil_andw_mem((uint32_t *)(TADR_RCU_BASE+TOFF_RCU_CTL), RCU_CTL_HXTALBPS);

    /*
	 *  PLLSEL, PREDV0_LSB, PLLMF, USBFSPSCZbg
	 */
	sil_andw_mem((uint32_t *)(TADR_RCU_BASE+TOFF_RCU_CFG0), RCU_CFG0_CONFIG2);
	sil_wrw_mem((uint32_t *)(TADR_RCU_BASE+TOFF_RCU_CFG1), 0);

    /*
	 *  HXTALEN, CKMEN, PLLEN, PLL1EN, PLL2ENZbg
	 */
	sil_andw_mem((uint32_t *)(TADR_RCU_BASE+TOFF_RCU_CTL), RCU_CTL_CONFIG2);

	/*
	 *  RCUݒ~
	 */
	sil_wrw_mem((uint32_t *)(TADR_RCU_BASE+TOFF_RCU_INT), 0x00FF0000);

    /*
	 * HXTALL
	 */
	sil_orw_mem((uint32_t *)(TADR_RCU_BASE+TOFF_RCU_CTL), RCU_CTL_HXTALEN);

    do{	/* L҂ */
        timeout++;
		sil_dly_nse(1000);
        stab_flag = (sil_rew_mem((uint32_t *)(TADR_RCU_BASE+TOFF_RCU_CTL)) & RCU_CTL_HXTALSTB);
		if(timeout > HXTAL_STARTUP_TIMEOUT)
			return -1;
    }while((0U == stab_flag));


    /*
	 *  AHBSYSCLKɐݒ, APB2,APB1ݒ
	 */
	sil_orw_mem((uint32_t *)(TADR_RCU_BASE+TOFF_RCU_CFG0), RCU_AHB_CKSYS_DIV1);
	sil_orw_mem((uint32_t *)(TADR_RCU_BASE+TOFF_RCU_CFG0), RCU_APB2_CKAHB_DIV1);
	sil_orw_mem((uint32_t *)(TADR_RCU_BASE+TOFF_RCU_CFG0), RCU_APB1_CKAHB_DIV2);

    /*
	 *  CK_PLL = (CK_PREDIV0) * 27 = 108 MHz
	 */ 
	sil_andw_mem((uint32_t *)(TADR_RCU_BASE+TOFF_RCU_CFG0), (RCU_CFG0_PLLMF | RCU_CFG0_PLLMF_4));
	sil_orw_mem((uint32_t *)(TADR_RCU_BASE+TOFF_RCU_CFG0), (RCU_PLLSRC_HXTAL | RCU_PLL_MUL27));

	sil_andw_mem((uint32_t *)(TADR_RCU_BASE+TOFF_RCU_CFG1), (RCU_CFG1_PREDV0SEL | RCU_CFG1_PREDV1 | RCU_CFG1_PLL1MF | RCU_CFG1_PREDV0));
	sil_orw_mem((uint32_t *)(TADR_RCU_BASE+TOFF_RCU_CFG1), (RCU_PREDV0SRC_HXTAL | RCU_PREDV0_DIV2 | RCU_PREDV1_DIV2 | RCU_PLL1_MUL20 | RCU_PLL2_MUL20));

	/*
	 *  PLL1L
	 */
	sil_orw_mem((uint32_t *)(TADR_RCU_BASE+TOFF_RCU_CTL), RCU_CTL_PLL1EN);
	timeout = 0;
	while(0U == (sil_rew_mem((uint32_t *)(TADR_RCU_BASE+TOFF_RCU_CTL)) & RCU_CTL_PLL1STB)){
		sil_dly_nse(1000);
		if(++timeout > HXTAL_STARTUP_TIMEOUT)
			return -1;
	}

	/*
	 *  PLL2L
	 */
	sil_orw_mem((uint32_t *)(TADR_RCU_BASE+TOFF_RCU_CTL), RCU_CTL_PLL2EN);
	timeout = 0;
	while(0U == (sil_rew_mem((uint32_t *)(TADR_RCU_BASE+TOFF_RCU_CTL)) & RCU_CTL_PLL2STB)){
		sil_dly_nse(1000);
		if(++timeout > HXTAL_STARTUP_TIMEOUT)
			return -1;
	}

	/*
	 *  PLLL
	 */
	sil_orw_mem((uint32_t *)(TADR_RCU_BASE+TOFF_RCU_CTL), RCU_CTL_PLLEN);
	timeout = 0;
    while(0U == (sil_rew_mem((uint32_t *)(TADR_RCU_BASE+TOFF_RCU_CTL)) & RCU_CTL_PLLSTB)){
		sil_dly_nse(1000);
		if(++timeout > HXTAL_STARTUP_TIMEOUT)
			return -1;
    }

    /*
	 *  VXeNbNPLLɐݒ
	 */
	sil_andw_mem((uint32_t *)(TADR_RCU_BASE+TOFF_RCU_CFG0), RCU_CFG0_SCS);
	sil_orw_mem((uint32_t *)(TADR_RCU_BASE+TOFF_RCU_CFG0), RCU_CKSYSSRC_PLL);

    /*
	 *  VXeNbNݒ҂
	 */
	timeout = 0;
    while(0U == (sil_rew_mem((uint32_t *)(TADR_RCU_BASE+TOFF_RCU_CFG0)) & RCU_SCSS_PLL)){
		sil_dly_nse(1000);
		if(++timeout > HXTAL_STARTUP_TIMEOUT)
			return -1;
    }
	return 0;
}

/*
 *  n[hEFAݒ
 */
void BeforeInit(void)
{
	/*
	 *  VXeNbNݒ
	 */
	if(SystemInit() != 0){
		while(1);
	}

	/*
	 *  ECLIC
	 */
	eclic_init();
	eclic_mode_enable();

	/*
	 *  CPU frequency𑪒
	 */
	measure_cpu_freq(1);
	cpu_frequency = measure_cpu_freq(100);

#ifndef USE_MCYCLE
    /*
	 *  ȃGlpmcycle~
	 */
	disable_mcycle_minstret();
#endif
}

/*
 *  RCUݒNbN擾
 *  parameter1: RCU_ClocksTypeDef\̂ւ̃|C^[
 *  clock frequency of system, AHB, APB1, APB2ݒ
 */
void RCC_GetClocksFreq(RCU_ClocksTypeDef *pclk)
{
	uint32_t cfg0 = sil_rew_mem((uint32_t *)(TADR_RCU_BASE+TOFF_RCU_CFG0));
	uint32_t cfg1 = sil_rew_mem((uint32_t *)(TADR_RCU_BASE+TOFF_RCU_CFG1));
    uint32_t cksys_freq;
    uint32_t pllsel, predv0sel, pllmf,ck_src, idx, clk_exp;
    uint32_t predv0, predv1, pll1mf;

	switch(cfg0 & RCU_CFG0_SCSS){
	case SEL_IRC8M:	/* IRC8M is selected as CK_SYS */
		cksys_freq = IRC8M_VALUE;
		break;
	case SEL_HXTAL:	/* HXTAL is selected as CK_SYS */
		cksys_freq = HXTAL_VALUE;
		break;
    case SEL_PLL:	/* PLL is selected as CK_SYS */
		/* PLL clock source selection, HXTAL or IRC8M/2 */
		pllsel = (cfg0 & RCU_CFG0_PLLSEL);

		if(RCU_PLLSRC_HXTAL == pllsel){
			ck_src = HXTAL_VALUE;	/* PLL clock source is HXTAL */

			predv0sel = (cfg1 & RCU_CFG1_PREDV0SEL);
			if(RCU_PREDV0SRC_CKPLL1 == predv0sel){	/* source clock use PLL1 */
				predv1 = (uint32_t)((cfg1 & RCU_CFG1_PREDV1) >> 4) + 1U;
				pll1mf = (uint32_t)((cfg1 & RCU_CFG1_PLL1MF) >> 8) + 2U;
				if(17U == pll1mf){
					pll1mf = 20U;
				}
				ck_src = (ck_src / predv1) * pll1mf;
			}
			predv0 = (cfg1 & RCU_CFG1_PREDV0) + 1U;
			ck_src /= predv0;
		}
		else{	/* PLL clock source is IRC8M/2 */
            ck_src = IRC8M_VALUE/2U;
		}

		/* PLL multiplication factor */
		pllmf = (cfg0 & RCU_CFG0_PLLMF) >> 18;
		if((cfg0 & RCU_CFG0_PLLMF_4))
			pllmf |= 0x10U;
		if(pllmf < 15U)
			pllmf += 2U;
		else
			pllmf += 1U;

		cksys_freq = ck_src * pllmf;

		if(15U == pllmf){
			/* PLL source clock multiply by 6.5 */
			cksys_freq = ck_src * 6U + ck_src / 2U;
        }
		break;
	/* IRC8M is selected as CK_SYS */
	default:
		cksys_freq = IRC8M_VALUE;
		break;
	}

	/* calculate AHB clock frequency */
	idx = (cfg0 & RCU_CFG0_AHBPSC) >> 4;
	clk_exp = ahb_exp[idx];
	pclk->AHB_Frequency = cksys_freq >> clk_exp;

    /* calculate APB1 clock frequency */
	idx = (cfg0 & RCU_CFG0_APB1PSC) >> 8;
	clk_exp = apb1_exp[idx];
	pclk->APB1_Frequency = pclk->AHB_Frequency >> clk_exp;

	/* calculate APB2 clock frequency */
	idx = (cfg0 & RCU_CFG0_APB2PSC) >> 11;
	clk_exp = apb2_exp[idx];
	pclk->APB2_Frequency = pclk->AHB_Frequency >> clk_exp;
	pclk->SYSCLK_Frequency = cksys_freq;
}

