/*
 *  rommon Simple Monitor
 * 
 *  Copyright (C) 1999-2004 by Dep. of Computer Science and Engineering
 *                   Tomakomai National College of Technology, JAPAN
 *  Copyright (C) 2015-2020 by Roi Takeuchi
 *
 *  L쌠҂́Cȉ (1)`(4) ̏CFree Software Foundation 
 *  ɂČ\Ă GNU General Public License  Version 2 ɋL
 *  qĂ𖞂ꍇɌC{\tgEFAi{\tgEFA
 *  ς̂܂ށDȉjgpEEρEĔzziȉC
 *  pƌĂԁj邱Ƃ𖳏ŋD
 *  (1) {\tgEFA\[XR[ȟ`ŗpꍇɂ́CL̒
 *      \C̗pщL̖ۏ؋K肪Ĉ܂܂̌`Ń\[
 *      XR[hɊ܂܂Ă邱ƁD
 *  (2) {\tgEFACCu`ȂǁC̃\tgEFAJɎg
 *      pł`ōĔzzꍇɂ́CĔzzɔhLgip
 *      ҃}jAȂǁjɁCL̒쌠\C̗pщL
 *      ̖ۏ؋Kfڂ邱ƁD
 *  (3) {\tgEFAC@ɑgݍނȂǁC̃\tgEFAJɎg
 *      płȂ`ōĔzzꍇɂ́Ĉꂩ̏𖞂
 *      ƁD
 *    (a) ĔzzɔhLgip҃}jAȂǁjɁCL̒
 *        쌠\C̗pщL̖ۏ؋Kfڂ邱ƁD
 *    (b) Ĕzž`ԂCʂɒ߂@ɂāCTOPPERSvWFNg
 *        񍐂邱ƁD
 *  (4) {\tgEFA̗pɂ蒼ړI܂͊ԐړIɐ邢Ȃ鑹
 *      QCL쌠҂TOPPERSvWFNgƐӂ邱ƁD
 *
 *  {\tgEFÁCۏ؂Œ񋟂Ă̂łDL쌠҂
 *  TOPPERSvWFNǵC{\tgEFAɊւāC̓Kp\
 *  ܂߂āCȂۏ؂sȂD܂C{\tgEFA̗pɂ蒼
 *  ړI܂͊ԐړIɐȂ鑹QɊւĂC̐ӔC𕉂ȂD
 * 
 *  @(#) $Id: $
 */

#include "sys_defs.h"
#include "sil.h"
#include <math.h>

#define sil_orw_mem(a, b)		sil_wrw_mem((a), sil_rew_mem(a) | (b))
#define sil_andw_mem(a, b)		sil_wrw_mem((a), sil_rew_mem(a) & ~(b))
#define sil_modw_mem(a, b, c)	sil_wrw_mem((a), (sil_rew_mem(a) & (~b)) | (c))

/*
 *  TIMEOUT VALUES
 */
#define CLOCKSWITCH_TIMEOUT_VALUE    (5000*1000)	/* 5 s    */ 
#define HSE_TIMEOUT_VALUE            (5000*1000)	/* Time out for HSE start up, in ms */
#define RCC_LSE_TIMEOUT_VALUE        (5000*1000)
#define PLL_TIMEOUT_VALUE            (2*1000)		/* 2 ms */
#define PLLSAI_TIMEOUT_VALUE         (100*1000)		/* Timeout value fixed to 100 ms  */
#define RCC_DBP_TIMEOUT_VALUE        (100*1000)		/* Timeout value fixed to 100 ms  */
#define HSI48_TIMEOUT_VALUE          (2*1000)		/* 2 ms */
#define PWR_OVERDRIVE_TIMEOUT_VALUE  (1000*1000)
#define PWR_FLAG_SETTING_DELAY_US    1000


#define REQUEST_SYSCLOCK             RCC_CFGR_SW_PLL1
#define REQUEST_FLASH_LATENCY        FLASH_ACR_LATENCY_4WS
#define REQUEST_PWR_SUPPLY           PWR_CR3_LDOEN
#define REQUEST_SYSCLK_DIV           RCC_D1CFGR_D1CPRE_DIV1
#define REQUEST_HCLK_DIV             RCC_D1CFGR_HPRE_DIV2
#define REQUEST_APB1_DIV             RCC_D2CFGR_D2PPRE1_DIV2
#define REQUEST_APB2_DIV             RCC_D2CFGR_D2PPRE2_DIV2
#define REQUEST_APB3_DIV             RCC_D1CFGR_D1PPRE_DIV2
#define REQUEST_APB4_DIV             RCC_D3CFGR_D3PPRE_DIV2

#define REQUEST_PLL_SOURCE           RCC_PLLCKSELR_PLLSRC_HSE
#define REQUEST_PLLM                 4
#define REQUEST_PLLN                 400
#define REQUEST_PLLP                 2
#define REQUEST_PLLR                 2
#define REQUEST_PLLQ                 4
#define REQUEST_PLLFRACN             0
#define REQUEST_PLL1VCOWIDE          0x00000000
#define REQUEST_PLL1VCIRANGE         RCC_PLLCFGR_PLL1RGE_2

#ifdef USE_USB
#define REQUEST_PLL3_PLL3M           8
#define REQUEST_PLL3_PLL3N           336
#define REQUEST_PLL3_PLL3FRACN       0
#define REQUEST_PLL3_PLL3P           2
#define REQUEST_PLL3_PLL3R           2
#define REQUEST_PLL3_PLL3Q           7
#define REQUEST_PLL3_PLL3VCOSEL      RCC_PLLCFGR_PLL3VCOSEL
#define REQUEST_PLL3_PLL3RGE         RCC_PLLCFGR_PLL3RGE_0
#define REQUEST_USBCLOCK             RCC_USBCLKSOURCE_PLL3
#endif

#define CFGR_SWS_SHIFT               3
#define D1CFGR_HPRE_SHIFT            0
#define D1CFGR_D1CPRE_SHIFT          8
#define D2CFGR_D2PPRE1_SHIFT         4
#define D2CFGR_D2PPRE2_SHIFT         8
#define D3CFGR_D3PPRE_SHIFT          4

#define REQUEST_VOLTAGE_SCALE        (PWR_D3CR_VOS_1 | PWR_D3CR_VOS_0)
#define PWR_SUPPLY_CONFIG_MASK       (PWR_CR3_SCUEN | PWR_CR3_LDOEN | PWR_CR3_BYPASS)

/*
 *  USBNbNݒ
 */
#define RCC_USBCLKSOURCE_PLL         RCC_D2CCIP2R_USBSEL_0
#define RCC_USBCLKSOURCE_PLL3        RCC_D2CCIP2R_USBSEL_1
#define RCC_USBCLKSOURCE_HSI48       RCC_D2CCIP2R_USBSEL

/*
 *  CORTEX MPÜ拖֎~
 */
#define MPU_REGION_ENABLE            0x01
#define MPU_REGION_DISABLE           0x00

/*
 *  CORTEX MPU CXgNVANZX֎~
 */
#define MPU_INSTRUCTION_ACCESS_ENABLE  0x00
#define MPU_INSTRUCTION_ACCESS_DISABLE 0x01

/*
 *  CORTEX MPU CXgNVANZXLbVݒ
 */
#define MPU_ACCESS_CACHEABLE         0x01
#define MPU_ACCESS_NOT_CACHEABLE     0x00

/*
 *  CORTEX MPU CXgNVANZXobt@ݒ
 */
#define MPU_ACCESS_BUFFERABLE        ((uint8_t)0x01)
#define MPU_ACCESS_NOT_BUFFERABLE    ((uint8_t)0x00)

/*
 *  CORTEX MPU CXgNVANZXLݒ
 */
#define MPU_ACCESS_SHAREABLE         ((uint8_t)0x01)
#define MPU_ACCESS_NOT_SHAREABLE     ((uint8_t)0x00)

/*
 *  CRS VNݒ
 */
#define RCC_CRS_SYNC_DIV1            (0x00000000U)			/* Synchro Signal not divided (default) */
#define RCC_CRS_SYNC_DIV2            CRS_CFGR_SYNCDIV_0		/* Synchro Signal divided by 2 */
#define RCC_CRS_SYNC_DIV4            CRS_CFGR_SYNCDIV_1		/* Synchro Signal divided by 4 */
#define RCC_CRS_SYNC_DIV8            (CRS_CFGR_SYNCDIV_1 | CRS_CFGR_SYNCDIV_0)	/* Synchro Signal divided by 8 */
#define RCC_CRS_SYNC_DIV16           CRS_CFGR_SYNCDIV_2		/* Synchro Signal divided by 16 */
#define RCC_CRS_SYNC_DIV32           (CRS_CFGR_SYNCDIV_2 | CRS_CFGR_SYNCDIV_0)	/* Synchro Signal divided by 32 */
#define RCC_CRS_SYNC_DIV64           (CRS_CFGR_SYNCDIV_2 | CRS_CFGR_SYNCDIV_1)	/* Synchro Signal divided by 64 */
#define RCC_CRS_SYNC_DIV128          CRS_CFGR_SYNCDIV		/* Synchro Signal divided by 128 */

/*
 *  CRS VN\[Xݒ
 */
#define RCC_CRS_SYNC_SOURCE_PIN      (0x00000000U)			/* Synchro Signal source external pin, Available on STM32H7 Rev.B and above devices only */
#define RCC_CRS_SYNC_SOURCE_LSE      CRS_CFGR_SYNCSRC_0		/* Synchro Signal source LSE */
#define RCC_CRS_SYNC_SOURCE_USB1     CRS_CFGR_SYNCSRC_1		/* Synchro Signal source USB1 SOF (default) */
#define RCC_CRS_SYNC_SOURCE_USB2     (CRS_CFGR_SYNCSRC_1|CRS_CFGR_SYNCSRC_0)	/* Synchro Signal source USB2 SOF */

/*
 *  CRS VNPolarityݒ
 */
#define RCC_CRS_SYNC_POLARITY_RISING  (0x00000000U)			/* Synchro Active on rising edge (default) */
#define RCC_CRS_SYNC_POLARITY_FALLING CRS_CFGR_SYNCPOL		/* Synchro Active on falling edge */

/*
 *  CRS VN[hlݒ
 */
#define RCC_CRS_RELOADVALUE_DEFAULT   (0x0000BB7FU)			/* The reset value of the RELOAD field corresponds
                                                                    to a target frequency of 48 MHz and a synchronization signal frequency of 1 kHz (SOF signal from USB). */

/*
 *  CRS HSI48Lu[VftHg
 */
#define RCC_CRS_HSI48CALIBRATION_DEFAULT (0x20U<<8)			/* The default value is 32, which corresponds to the middle of the trimming interval.
                                                                      The trimming step is around 67 kHz between two consecutive TRIM steps. A higher TRIM value
                                                                      corresponds to a higher output frequency */

#define RCC_CRS_ERRORLIMIT_DEFAULT    (0x22U<<16)			/* Default Frequency error limit */


/*
 *  LbVTCYID WX^}N
 */
#define CCSIDR_WAYS(x)         (((x) & SCB_CCSIDR_ASSOCIATIVITY_Msk) >> SCB_CCSIDR_ASSOCIATIVITY_Pos)
#define CCSIDR_SETS(x)         (((x) & SCB_CCSIDR_NUMSETS_Msk      ) >> SCB_CCSIDR_NUMSETS_Pos      )


#define CR_RESET_VALUE         (RCC_CR_PLL3ON   | RCC_CR_PLL2ON | RCC_CR_PLL1ON  | \
                                RCC_CR_CSSHSEON | RCC_CR_HSEON  | RCC_CR_HSI48ON | \
                                RCC_CR_CSIKERON | RCC_CR_CSION)

#define get_sysclk_source()    (sil_rew_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_CFGR)) & RCC_CFGR_SWS)
#define get_pll1n_divider()    (sil_rew_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_PLL1DIVR)) & RCC_PLL1DIVR_N1)
#define get_pll2n_divider()    (sil_rew_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_PLL2DIVR)) & RCC_PLL2DIVR_N2)
#define get_pll3n_divider()    (sil_rew_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_PLL3DIVR)) & RCC_PLL3DIVR_N3)
#define get_hsi_divider()      ((sil_rew_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_CR)) & RCC_CR_HSIDIV) >> 3)


static void MPU_Config(void);
static int  SystemClock_Config(void);
static int  Set_Flash_latency(uint32_t latency);

const uint8_t D1CorePrescTable[16] = {0, 0, 0, 0, 1, 2, 3, 4, 1, 2, 3, 4, 6, 7, 8, 9};

uint32_t SystemCoreClock, SystemD2Clock;

/*
 *  VXeNbN擾
 */
static uint32_t RCC_GetSysClockFreq(void)
{
	uint32_t pllp, pllsource, pllm, pllfracen, hsivalue;
	float_t fracn1, pllvco;
	uint32_t sysclockfreq, pllckselr;

	/* Get SYSCLK source -------------------------------------------------------*/

	switch(sil_rew_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_CFGR)) & RCC_CFGR_SWS){
	case RCC_CFGR_SWS_HSI:  /* HSI used as system clock source */
		if((sil_rew_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_CR)) & RCC_CR_HSIDIVF) != 0)
			sysclockfreq = HSI_VALUE >> get_hsi_divider();
		else
			sysclockfreq = HSI_VALUE;
	break;

	case RCC_CFGR_SWS_HSE:  /* HSE used as system clock  source */
		sysclockfreq = HSE_VALUE;
		break;

	case RCC_CFGR_SWS_PLL1:  /* PLL1 used as system clock  source */
		/* PLL_VCO = (HSE_VALUE or HSI_VALUE or CSI_VALUE/ PLLM) * PLLN
		 * SYSCLK = PLL_VCO / PLLR
		 */
		pllckselr = sil_rew_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_PLLCKSELR));
		pllsource = (pllckselr & RCC_PLLCKSELR_PLLSRC);
		pllm = ((pllckselr & RCC_PLLCKSELR_DIVM1)>> 4);
		pllfracen = pllckselr & RCC_PLLCFGR_PLL1FRACEN;
		fracn1 = (float_t)(uint32_t)(pllfracen* ((sil_rew_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_PLL1FRACR)) & RCC_PLL1FRACR_FRACN1)>> 3));

		if(pllm != 0U){
			switch (pllsource){
			case RCC_PLLCKSELR_PLLSRC_HSI:	/* HSI used as PLL clock source */
				if((sil_rew_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_CR)) & RCC_CR_HSIDIVF) != 0){
					hsivalue= HSI_VALUE >> get_hsi_divider();
					pllvco = ( (float_t)hsivalue / (float_t)pllm) * ((float_t)get_pll1n_divider() + (fracn1/8192.0) + 1.0);
				}
				else
					pllvco = ((float_t)HSI_VALUE / (float_t)pllm) * ((float_t)get_pll1n_divider() + (fracn1/8192.0) + 1.0);
				break;
			case RCC_PLLCKSELR_PLLSRC_HSE:	/* HSE used as PLL clock source */
				pllvco = ((float_t)HSE_VALUE / (float_t)pllm) * ((float_t)get_pll1n_divider() + (fracn1/8192.0) + 1.0);
				break;
			default:
				pllvco = ((float_t)CSI_VALUE / (float_t)pllm) * ((float_t)get_pll1n_divider() + (fracn1/8192.0) + 1.0);
				break;
			}
			pllp = (((sil_rew_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_PLL1DIVR)) & RCC_PLL1DIVR_P1) >> 9) + 1);
			sysclockfreq =  (uint32_t)(float_t)(pllvco/(float_t)pllp);
		}
		else
			sysclockfreq = 0;
		break;

	default:
		sysclockfreq = CSI_VALUE;
		break;
	}
	return sysclockfreq;
}

/*
 *  PLL2NbN擾
 *  0-P, 1-Q, 2-R
 *  PLL_VCO = (HSE_VALUE or HSI_VALUE or CSI_VALUE/ PLL2M) * PLL2N
 *  PLL2xCLK = PLL2_VCO / PLL2x
 */
static void RCC_GetPLL2ClockFreq(uint32_t *PLL2_Clocks)
{
	uint32_t  pllsource, pll2m,  pll2fracen, hsivalue, pll2divr;
	float_t fracn2, pll2vco;
	uint32_t pllckselr = sil_rew_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_PLLCKSELR));

	pllsource = pllckselr & RCC_PLLCKSELR_PLLSRC;
	pll2m = (pllckselr & RCC_PLLCKSELR_DIVM2)>> 12;
	pll2fracen = sil_rew_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_PLLCFGR)) & RCC_PLLCFGR_PLL2FRACEN;
	fracn2 =(float_t)(uint32_t)(pll2fracen * ((sil_rew_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_PLL2FRACR)) & RCC_PLL2FRACR_FRACN2) >> 3));

	if(pll2m != 0U){
		switch (pllsource){
		case RCC_PLLCKSELR_PLLSRC_HSI:	/* HSI used as PLL clock source */
			if((sil_rew_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_CR)) & RCC_CR_HSIDIVF) != 0){
				hsivalue = HSI_VALUE >> get_hsi_divider();
				pll2vco = ((float_t)hsivalue / (float_t)pll2m) * ((float_t)get_pll2n_divider() + (fracn2/8192.0) + 1.0);
			}
			else
				pll2vco = ((float_t)HSI_VALUE / (float_t)pll2m) * ((float_t)get_pll2n_divider() + (fracn2/8192.0) + 1.0);
			break;
		case RCC_PLLCKSELR_PLLSRC_HSE:	/* HSE used as PLL clock source */
			pll2vco = ((float_t)HSE_VALUE / (float_t)pll2m) * ((float_t)get_pll2n_divider() + (fracn2/8192.0) + 1.0);
			break;
		default:
			pll2vco = ((float_t)CSI_VALUE / (float_t)pll2m) * ((float_t)get_pll2n_divider() + (fracn2/8192.0) + 1.0);
			break;
		}
		pll2divr = sil_rew_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_PLL2DIVR));
		PLL2_Clocks[0] = (uint32_t)(float_t)(pll2vco/((float_t)(uint32_t)((pll2divr & RCC_PLL2DIVR_P2) >>9)  + 1.0));
		PLL2_Clocks[1] = (uint32_t)(float_t)(pll2vco/((float_t)(uint32_t)((pll2divr & RCC_PLL2DIVR_Q2) >>16) + 1.0));
		PLL2_Clocks[2] = (uint32_t)(float_t)(pll2vco/((float_t)(uint32_t)((pll2divr & RCC_PLL2DIVR_R2) >>24) + 1.0));
	}
	else{
		PLL2_Clocks[0] = 0;
		PLL2_Clocks[1] = 0;
		PLL2_Clocks[2] = 0;
	}
}

/*
 *  PLL3NbN擾
 *  0-P, 1-Q, 2-R
 *  PLL3_VCO = (HSE_VALUE or HSI_VALUE or CSI_VALUE/ PLL3M) * PLL3N
 *  PLL3xCLK = PLL3_VCO / PLLxR
 */
static void RCC_GetPLL3ClockFreq(uint32_t *PLL3_Clocks)
{
	uint32_t pllsource, pll3m, pll3fracen, hsivalue, pll3divr;
	float_t fracn3, pll3vco;
	uint32_t pllckselr = sil_rew_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_PLLCKSELR));

	pllsource = pllckselr & RCC_PLLCKSELR_PLLSRC;
	pll3m = (pllckselr & RCC_PLLCKSELR_DIVM3) >> 20;
	pll3fracen = sil_rew_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_PLLCFGR)) & RCC_PLLCFGR_PLL3FRACEN;
	fracn3 = (float_t)(uint32_t)(pll3fracen* ((sil_rew_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_PLL3FRACR)) & RCC_PLL3FRACR_FRACN3) >> 3));

	if(pll3m != 0U){
		switch (pllsource){
		case RCC_PLLCKSELR_PLLSRC_HSI:	/* HSI used as PLL clock source */
			if((sil_rew_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_CR)) & RCC_CR_HSIDIVF) != 0){
				hsivalue = HSI_VALUE >> get_hsi_divider();
				pll3vco = ((float_t)hsivalue / (float_t)pll3m) * ((float_t)get_pll3n_divider() + (fracn3/8192.0) + 1.0);
			}
			else
				pll3vco = ((float_t)HSI_VALUE / (float_t)pll3m) * ((float_t)get_pll3n_divider() + (fracn3/8192.0) + 1.0);
			break;
		case RCC_PLLCKSELR_PLLSRC_HSE:	/* HSE used as PLL clock source */
			pll3vco = ((float_t)HSE_VALUE / (float_t)pll3m) * ((float_t)get_pll3n_divider() + (fracn3/8192.0) + 1.0);
			break;
		default:
			pll3vco = ((float_t)CSI_VALUE / (float_t)pll3m) * ((float_t)get_pll3n_divider() + (fracn3/8192.0) + 1.0);
			break;
		}
		pll3divr = sil_rew_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_PLL3DIVR));
		PLL3_Clocks[0] = (uint32_t)(float_t)(pll3vco/((float_t)(uint32_t)((pll3divr & RCC_PLL3DIVR_P3) >>9)  + 1.0));
		PLL3_Clocks[1] = (uint32_t)(float_t)(pll3vco/((float_t)(uint32_t)((pll3divr & RCC_PLL3DIVR_Q3) >>16) + 1.0));
		PLL3_Clocks[2] = (uint32_t)(float_t)(pll3vco/((float_t)(uint32_t)((pll3divr & RCC_PLL3DIVR_R3) >>24) + 1.0));
	}
	else{
		PLL3_Clocks[0] = 0;
		PLL3_Clocks[1] = 0;
		PLL3_Clocks[2] = 0;
	}
}


/*
 *  Setup the microcontroller system
 *     Initialize the Embedded Flash Interface, the PLL and update the 
 *     SystemFrequency variable.
 */
void SystemInit(void)
{
	uint32_t ccsidr, sets, ways;

	/*
	 *  FPU̗L
	 */
	sil_orw_mem((uint32_t *)(TADR_SCB_BASE+TOFF_SCB_CPACR), ((3 << (10*2))|(3 << (11*2))));

	/*
	 *  HSIL
	 */
	sil_orw_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_CR), RCC_CR_HSION);

	/*
	 *  CFGRWX^
	 */
	sil_wrw_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_CFGR), 0x00000000);

	/*
	 *  HSIȊÕNbN~
	 */
	sil_andw_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_CR), CR_RESET_VALUE);

	/*
	 *  RCCWX^̃Zbg
	 */
	sil_wrw_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_D1CFGR), 0x00000000);
	sil_wrw_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_D2CFGR), 0x00000000);
	sil_wrw_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_D3CFGR), 0x00000000);
	sil_wrw_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_PLLCKSELR), 0x00000000);
	sil_wrw_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_PLLCFGR), 0x00000000);
	sil_wrw_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_PLL1DIVR), 0x00000000);
	sil_wrw_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_PLL1FRACR), 0x00000000);
	sil_wrw_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_PLL2DIVR), 0x00000000);
	sil_wrw_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_PLL2FRACR), 0x00000000);
	sil_wrw_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_PLL3DIVR), 0x00000000);
	sil_wrw_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_PLL3FRACR), 0x00000000);
	sil_andw_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_CR), RCC_CR_HSEBYP);

	/*
	 *  RCC݂֎~
	 */
	sil_wrw_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_CIER), 0x00000000);

	/* Change  the switch matrix read issuing capability to 1 for the AXI SRAM target (Target 7) */
	if((sil_rew_mem((uint32_t *)(TADR_DBGMCU_BASE+TOFF_DBGMCU_IDCODE)) & 0xFFFF0000U) < 0x20000000U){
		/* if stm32h7 revY*/
		/* Change  the switch matrix read issuing capability to 1 for the AXI SRAM target (Target 7) */
		sil_wrw_mem((uint32_t *)0x51008108, 0x000000001);
	}

	/*
	 *  veNVjbg̏
	 */
	MPU_Config();

	/*
	 *  I-CacheL
	 */
	asm volatile ("dsb 0xF":::"memory");
	asm volatile ("isb 0xF":::"memory");
	sil_wrw_mem((uint32_t *)(TADR_SCB_BASE+TOFF_SCB_ICIALLU), 0);	/* invalidate I-Cache */
	asm volatile ("dsb 0xF":::"memory");
	asm volatile ("isb 0xF":::"memory");
	sil_orw_mem((uint32_t *)(TADR_SCB_BASE+TOFF_SCB_CCR), SCB_CCR_IC_Msk);	/* enable I-Cache */
	asm volatile ("dsb 0xF":::"memory");
	asm volatile ("isb 0xF":::"memory");

	/*
	 *  D-CacheL
	 */
	sil_wrw_mem((uint32_t *)(TADR_SCB_BASE+TOFF_SCB_CSSELR), ((0 << 1) | 0));	/* Level 1 data cache */
	ccsidr  = sil_rew_mem((uint32_t *)(TADR_SCB_BASE+TOFF_SCB_CCSIDR));
    sets    = (uint32_t)(CCSIDR_SETS(ccsidr));
	asm volatile ("dsb 0xF":::"memory");

	do{                                   // invalidate D-Cache
		ways = (uint32_t)(CCSIDR_WAYS(ccsidr));
		do{
			sil_wrw_mem((uint32_t *)(TADR_SCB_BASE+TOFF_SCB_DCISW),
				(((sets << SCB_DCISW_SET_Pos) & SCB_DCISW_SET_Msk) | \
				((ways << SCB_DCISW_WAY_Pos) & SCB_DCISW_WAY_Msk)));
        }while(ways--);
    }while(sets--);

	asm volatile ("dsb 0xF":::"memory");

	sil_orw_mem((uint32_t *)(TADR_SCB_BASE+TOFF_SCB_CCR), SCB_CCR_DC_Msk);	/* enable D-Cache */
	asm volatile ("dsb 0xF":::"memory");
	asm volatile ("isb 0xF":::"memory");

	/*
	 *  VXeNbÑRtBM[V
	 */
	if(SystemClock_Config() != 0){
		for(;;){}
	}
}

/*
 *  FLASH-ROMLatencyݒ肷
 */
static int Set_Flash_latency(uint32_t latency)
{
	sil_modw_mem((uint32_t *)(TADR_FLASH_R_BASE+TOFF_FLASH_ACR), FLASH_ACR_LATENCY, latency);

	/*
	 *  Latency̐ݒ𔻒
	 */
	if((sil_rew_mem((uint32_t *)(TADR_FLASH_R_BASE+TOFF_FLASH_ACR)) & FLASH_ACR_LATENCY) != latency)
		return -1;
	else
		return 0;
}

/*
 *  Configure the MPU attributes as Write Through for SRAM1/2.
 *  note   The Base Address is 0x20010000 since this memory interface is the AXI.
 *         The Region Size is 256KB, it is related to SRAM1 and SRAM2  memory size.
 */
static void MPU_Config(void)
{
	/* Disable the MPU */
	sil_andw_mem((uint32_t *)(TADR_SCB_BASE+TOFF_SCB_SHCSR), SCB_SHCSR_MEMFAULTENA_Msk);
	sil_andw_mem((uint32_t *)(TADR_MPU_BASE+TOFF_MPU_CTRL), MPU_CTRL_ENABLE_Msk);

	/* Set the Region number */
	sil_wrw_mem((uint32_t *)(TADR_MPU_BASE+TOFF_MPU_RNR), MPU_REGION_NUMBER0);
	sil_wrw_mem((uint32_t *)(TADR_MPU_BASE+TOFF_MPU_RBAR), 0x30040000);
	sil_wrw_mem((uint32_t *)(TADR_MPU_BASE+TOFF_MPU_RASR),
				((MPU_INSTRUCTION_ACCESS_ENABLE << MPU_RASR_XN_Pos)   |
                 (MPU_REGION_FULL_ACCESS        << MPU_RASR_AP_Pos)   |
                 (MPU_TEX_LEVEL0                << MPU_RASR_TEX_Pos)  |
                 (MPU_ACCESS_NOT_SHAREABLE      << MPU_RASR_S_Pos)    |
                 (MPU_ACCESS_NOT_CACHEABLE      << MPU_RASR_C_Pos)    |
                 (MPU_ACCESS_BUFFERABLE         << MPU_RASR_B_Pos)    |
                 (0x00                          << MPU_RASR_SRD_Pos)  |
                 (MPU_REGION_SIZE_32KB          << MPU_RASR_SIZE_Pos) |
                 (MPU_REGION_ENABLE             << MPU_RASR_ENABLE_Pos)));

	/* Enable the MPU */
	sil_wrw_mem((uint32_t *)(TADR_MPU_BASE+TOFF_MPU_CTRL), (MPU_PRIVILEGED_DEFAULT | MPU_CTRL_ENABLE_Msk));
	sil_orw_mem((uint32_t *)(TADR_SCB_BASE+TOFF_SCB_SHCSR), SCB_SHCSR_MEMFAULTENA_Msk);
}


/*
 *  System Clock Configuration
 *         The system Clock is configured as follow : 
 *            System Clock source            = PLL (HSE BYPASS)
 *            SYSCLK(Hz)                     = 400000000 (CPU Clock)
 *            HCLK(Hz)                       = 200000000 (AXI and AHBs Clock)
 *            AHB Prescaler                  = 2
 *            D1 APB3 Prescaler              = 2 (APB3 Clock  100MHz)
 *            D2 APB1 Prescaler              = 2 (APB1 Clock  100MHz)
 *            D2 APB2 Prescaler              = 2 (APB2 Clock  100MHz)
 *            D3 APB4 Prescaler              = 2 (APB4 Clock  100MHz)
 *            HSE Frequency(Hz)              = 8000000
 *            PLL_M                          = 4
 *            PLL_N                          = 400
 *            PLL_P                          = 2
 *            PLL_Q                          = 4
 *            PLL_R                          = 2
 *            VDD(V)                         = 3.3
 *            Flash Latency(WS)              = 4
 */
static int SystemClock_Config(void)
{
	uint32_t rccbase = TADR_RCC_BASE;
	uint32_t tick = 0, tmp;

	/*
	 *  p[TvC RtBM[VAbvf[g`FbN
	 */
	if((sil_rew_mem((uint32_t *)(TADR_PWR_BASE+TOFF_PWR_CR3)) & PWR_CR3_SCUEN) == 0){
		if((sil_rew_mem((uint32_t *)(TADR_PWR_BASE+TOFF_PWR_CR3)) & PWR_SUPPLY_CONFIG_MASK) != REQUEST_PWR_SUPPLY){
			/* bNԂȂΏI */
			return -1;
		}
	}

	/*
	 *  p[TvC RtBM[V
	 */
	sil_modw_mem((uint32_t *)(TADR_PWR_BASE+TOFF_PWR_CR3), PWR_SUPPLY_CONFIG_MASK, REQUEST_PWR_SUPPLY);

	/*
	 *  VOLTAGExtOI܂ő҂
	 */
	while((sil_rew_mem((uint32_t *)(TADR_PWR_BASE+TOFF_PWR_CSR1)) & PWR_CSR1_ACTVOSRDY) == 0){
		if(tick++ > PWR_FLAG_SETTING_DELAY_US){
			return -2;
		}
		sil_dly_nse(1000);
	}

	/*
	 *  őNbN\xp VOLTAGE SCALEݒs
	 */
	sil_andw_mem((uint32_t *)(TADR_SYSCFG_BASE+TOFF_SYSCFG_PWRCR), SYSCFG_PWRCR_ODEN);
	tmp  = sil_rew_mem((uint32_t *)(TADR_SYSCFG_BASE+TOFF_SYSCFG_PWRCR));

	sil_modw_mem((uint32_t *)(TADR_PWR_BASE+TOFF_PWR_D3CR), PWR_D3CR_VOS, REQUEST_VOLTAGE_SCALE);
	tmp = sil_rew_mem((uint32_t *)(TADR_PWR_BASE+TOFF_PWR_D3CR));
	(void)(tmp);

	while((sil_rew_mem((uint32_t *)(TADR_PWR_BASE+TOFF_PWR_D3CR)) & PWR_D3CR_VOSRDY) == 0){}

	/*
	 *  HSEݒ
	 */
	sil_orw_mem((uint32_t *)(rccbase+TOFF_RCC_CR), RCC_CR_HSEBYP);
	sil_orw_mem((uint32_t *)(rccbase+TOFF_RCC_CR), RCC_CR_HSEON);

	/*
	 *  HSEfB҂
	 */
	tick = 0;
	while((sil_rew_mem((uint32_t *)(rccbase+TOFF_RCC_CR)) & RCC_CR_HSERDY) == 0){
		if(tick++ > HSE_TIMEOUT_VALUE)
			return -2;
		sil_dly_nse(1000);
	}

#ifdef REQUEST_USBCLOCK
	/*
	 *  USBV3.3M[^
	 */
	sil_orw_mem((uint32_t *)(TADR_PWR_BASE+TOFF_PWR_CR3), PWR_CR3_USB33DEN);

#if REQUEST_USBCLOCK == RCC_USBCLKSOURCE_HSI48
	/*
	 *  HSI48NbNN
	 */
	sil_orw_mem((uint32_t *)(rccbase+TOFF_RCC_CR), RCC_CR_HSI48ON);

	/*
	 *  HSI48fB҂
	 */
	tick = 0;
	while((sil_rew_mem((uint32_t *)(rccbase+TOFF_RCC_CR)) & RCC_CR_HSI48RDY) == 0){
		if(tick++ > HSI48_TIMEOUT_VALUE)
			return -2;
		sil_dly_nse(1000);
	}

	/*
	 *  CRSN
	 */
	sil_orw_mem((uint32_t *)(rccbase+TOFF_RCC_APB1HENR), RCC_APB1HENR_CRSEN);
	tmp = sil_rew_mem((uint32_t *)(rccbase+TOFF_RCC_APB1HENR));
	sil_orw_mem((uint32_t *)(rccbase+TOFF_RCC_APB1HRSTR), RCC_APB1HRSTR_CRSRST);
	sil_andw_mem((uint32_t *)(rccbase+TOFF_RCC_APB1HRSTR), RCC_APB1HRSTR_CRSRST);

	/*
	 *  CRSRtBM[Vݒ
	 */
	tmp = (RCC_CRS_SYNC_DIV1 | RCC_CRS_SYNC_SOURCE_USB2 | RCC_CRS_SYNC_POLARITY_RISING);
	tmp |= RCC_CRS_RELOADVALUE_DEFAULT;	/* ReloadValue value */
	tmp |= RCC_CRS_ERRORLIMIT_DEFAULT;	/* ErrorLimitValue value */
	sil_wrw_mem((uint32_t *)(TADR_CRS_BASE+TOFF_CRS_CFGR), tmp);

	/*
	 *  CRSLu[Vlݒ
	 */
	sil_modw_mem((uint32_t *)(TADR_CRS_BASE+TOFF_CRS_CR), CRS_CR_TRIM, RCC_CRS_HSI48CALIBRATION_DEFAULT);

	/*
	 *  CRSX^[g
	 */
	sil_orw_mem((uint32_t *)(TADR_CRS_BASE+TOFF_CRS_CR), (CRS_CR_AUTOTRIMEN | CRS_CR_CEN));

#endif	/* RCC_USBCLKSOURCE_HSI48 */
#endif	/* USE_USB */

	/*
	 *  PLLݒ
	 */
    /*
	 *  PLLVXeNbNݒς݂̏ꍇAG[
	 */
	if(get_sysclk_source() == RCC_CFGR_SWS_PLL1)
		return -1;

	/*
	 *  PLL DISABLE
	 */
	sil_andw_mem((uint32_t *)(rccbase+TOFF_RCC_CR), RCC_CR_PLL1ON);

	/*
	 *  PLL NOT-READY҂
	 */
	tick = 0;
	while((sil_rew_mem((uint32_t *)(rccbase+TOFF_RCC_CR)) & RCC_CR_PLL1RDY) != 0){
		if( tick++ > PLL_TIMEOUT_VALUE)
			return -2;
		sil_dly_nse(1000);
	}

	/*
	 *  PLLM,N,P,R,Qݒ
	 */
	sil_modw_mem((uint32_t *)(rccbase+TOFF_RCC_PLLCKSELR),
				(RCC_PLLCKSELR_PLLSRC | RCC_PLLCKSELR_DIVM1), (REQUEST_PLL_SOURCE | (REQUEST_PLLM<<4)));
	sil_wrw_mem((uint32_t *)(rccbase+TOFF_RCC_PLL1DIVR),
				((REQUEST_PLLN-1) | ((REQUEST_PLLP-1) << 9) | ((REQUEST_PLLR-1) << 24) | ((REQUEST_PLLQ-1) << 16)));

	/*
	 *  PLLFRACN
	 */
	sil_andw_mem((uint32_t *)(rccbase+TOFF_RCC_PLLCFGR), RCC_PLLCFGR_PLL1FRACEN);

	/* Configure PLL  PLL1FRACN */
	sil_modw_mem((uint32_t *)(rccbase+TOFF_RCC_PLL1FRACR), RCC_PLL1FRACR_FRACN1, REQUEST_PLLFRACN);

	/* Select PLL1 input reference frequency range: VCI */
	sil_modw_mem((uint32_t *)(rccbase+TOFF_RCC_PLLCFGR), RCC_PLLCFGR_PLL1RGE, REQUEST_PLL1VCIRANGE);

	/* Select PLL1 output frequency range : VCO */
	sil_modw_mem((uint32_t *)(rccbase+TOFF_RCC_PLLCFGR), RCC_PLLCFGR_PLL1VCOSEL, REQUEST_PLL1VCOWIDE);

	/*
	 *  PLL1PNbNL
	 */
	sil_orw_mem((uint32_t *)(rccbase+TOFF_RCC_PLLCFGR), RCC_PLLCFGR_DIVP1EN);

	/*
	 *  PLL1QNbNL
	 */
	sil_orw_mem((uint32_t *)(rccbase+TOFF_RCC_PLLCFGR), RCC_PLLCFGR_DIVQ1EN);

	/*
	 *  PLL1RNbNL
	 */
	sil_orw_mem((uint32_t *)(rccbase+TOFF_RCC_PLLCFGR), RCC_PLLCFGR_DIVR1EN);

	/*
	 *  PLL1FRACNL
	 */
	sil_orw_mem((uint32_t *)(rccbase+TOFF_RCC_PLLCFGR), RCC_PLLCFGR_PLL1FRACEN);

	/*
	 *  PLLNbNL
	 */
	sil_orw_mem((uint32_t *)(rccbase+TOFF_RCC_CR), RCC_CR_PLL1ON);

	/*
	 *  PLLfB҂
	 */
	tick = 0;
	while((sil_rew_mem((uint32_t *)(rccbase+TOFF_RCC_CR)) & RCC_CR_PLL1RDY) == 0){
		if( tick++ > PLL_TIMEOUT_VALUE)
			return -2;
		sil_dly_nse(1000);
	}

#ifdef REQUEST_USBCLOCK
#if REQUEST_USBCLOCK == RCC_USBCLKSOURCE_PLL3
	/*
	 *  USBNbNݒ
	 *  PLL3~
	 */
	sil_andw_mem((uint32_t *)(rccbase+TOFF_RCC_CR), RCC_CR_PLL3ON);
	tick = 0;
	while((sil_rew_mem((uint32_t *)(rccbase+TOFF_RCC_CR)) & RCC_CR_PLL3RDY) != 0){
		if(tick++ > PLL_TIMEOUT_VALUE)
			return -2;
		sil_dly_nse(1000);
	}

    /*
	 *  PLL3̃RtBM[Vݒ
	 */
	sil_modw_mem((uint32_t *)(rccbase+TOFF_RCC_PLLCKSELR), RCC_PLLCKSELR_DIVM3, (REQUEST_PLL3_PLL3M << 20));
	sil_wrw_mem((uint32_t *)(rccbase+TOFF_RCC_PLL3DIVR), ((REQUEST_PLL3_PLL3N-1) & RCC_PLL3DIVR_N3) | \
						(((REQUEST_PLL3_PLL3P-1) << 9) & RCC_PLL3DIVR_P3) | (((REQUEST_PLL3_PLL3Q-1) << 16) & RCC_PLL3DIVR_Q3) | \
						(((REQUEST_PLL3_PLL3R-1) << 24) & RCC_PLL3DIVR_R3));

	/*
	 *  PLL3 VCIݒ
	 */
	sil_modw_mem((uint32_t *)(rccbase+TOFF_RCC_PLLCFGR), RCC_PLLCFGR_PLL3RGE, REQUEST_PLL3_PLL3RGE);

    /*
	 *  PLL3 VCOݒ
	 */
	sil_modw_mem((uint32_t *)(rccbase+TOFF_RCC_PLLCFGR), RCC_PLLCFGR_PLL3VCOSEL, REQUEST_PLL3_PLL3VCOSEL);

	/*
	 *  PLL3FRACN~
	 */
	sil_andw_mem((uint32_t *)(rccbase+TOFF_RCC_PLLCFGR), RCC_PLLCFGR_PLL3FRACEN);

	/*
	 *  PLL3NbN Fractional Part Of The Multiplication Factorݒ
	 */
	sil_modw_mem((uint32_t *)(rccbase+TOFF_RCC_PLL3FRACR), RCC_PLL3FRACR_FRACN3, REQUEST_PLL3_PLL3FRACN);

	/*
	 *  PLL3FRACNĊJ
	 */
	sil_orw_mem((uint32_t *)(rccbase+TOFF_RCC_PLLCFGR), RCC_PLLCFGR_PLL3FRACEN);

	/*
	 *  PLL3-QNbNo
	 */
	sil_orw_mem((uint32_t *)(rccbase+TOFF_RCC_PLLCFGR), RCC_PLLCFGR_DIVQ3EN);

	/*
	 *  PLL3L
	 */
	sil_orw_mem((uint32_t *)(rccbase+TOFF_RCC_CR), RCC_CR_PLL3ON);
	tick = 0;
	while((sil_rew_mem((uint32_t *)(rccbase+TOFF_RCC_CR)) & RCC_CR_PLL3RDY) == 0){
		if(tick++ > PLL_TIMEOUT_VALUE)
			return -2;
		sil_dly_nse(1000);
	}
#endif	/* RCC_USBCLKSOURCE_PLL3 */

	/*
	 *  USBݒ
	 */
	sil_modw_mem((uint32_t *)(rccbase+TOFF_RCC_D2CCIP2R), RCC_D2CCIP2R_USBSEL, REQUEST_USBCLOCK);
#endif	/* USE_USB */

	/*
	 *  FLASHANZXEFCgݒ(ݒlxꍇ)
	 */
	if(REQUEST_FLASH_LATENCY > (sil_rew_mem((uint32_t *)(TADR_FLASH_R_BASE+TOFF_FLASH_ACR)) & FLASH_ACR_LATENCY)){
		if(Set_Flash_latency(FLASH_ACR_LATENCY) != 0)
			return -3;
	}

	/*
	 *  D1PCLK1ݒ
	 */
    if(REQUEST_APB3_DIV > (sil_rew_mem((uint32_t *)(rccbase+TOFF_RCC_D1CFGR)) & RCC_D1CFGR_D1PPRE))
		sil_modw_mem((uint32_t *)(rccbase+TOFF_RCC_D1CFGR), RCC_D1CFGR_D1PPRE, REQUEST_APB3_DIV);

	/*
	 *  PCLK1ݒ
	 */
	if(REQUEST_APB1_DIV > (sil_rew_mem((uint32_t *)(rccbase+TOFF_RCC_D2CFGR)) & RCC_D2CFGR_D2PPRE1))
		sil_modw_mem((uint32_t *)(rccbase+TOFF_RCC_D2CFGR), RCC_D2CFGR_D2PPRE1, REQUEST_APB1_DIV);

	/*
	 *  PCLK2ݒ
	 */
	if(REQUEST_APB2_DIV > (sil_rew_mem((uint32_t *)(rccbase+TOFF_RCC_D2CFGR)) & RCC_D2CFGR_D2PPRE2))
		sil_modw_mem((uint32_t *)(rccbase+TOFF_RCC_D2CFGR), RCC_D2CFGR_D2PPRE2, REQUEST_APB2_DIV);

	/*
	 *  D3PCLK1ݒ
	 */
	if(REQUEST_APB4_DIV > (sil_rew_mem((uint32_t *)(rccbase+TOFF_RCC_D3CFGR)) & RCC_D3CFGR_D3PPRE))
		sil_modw_mem((uint32_t *)(rccbase+TOFF_RCC_D3CFGR), RCC_D3CFGR_D3PPRE, REQUEST_APB4_DIV);

	/*
	 *  HCLKݒ
	 */
	if(REQUEST_HCLK_DIV > (sil_rew_mem((uint32_t *)(rccbase+TOFF_RCC_D1CFGR)) & RCC_D1CFGR_HPRE))
		sil_modw_mem((uint32_t *)(rccbase+TOFF_RCC_D1CFGR), RCC_D1CFGR_HPRE, REQUEST_HCLK_DIV);

	/*
	 *  SYSCLKݒ
	 */
	sil_modw_mem((uint32_t *)(rccbase+TOFF_RCC_D1CFGR), RCC_D1CFGR_D1CPRE, REQUEST_SYSCLK_DIV);

	/* HSE is selected as System Clock Source */
	/* PLL is selected as System Clock Source */
	/* Check the PLL ready flag */
	if((sil_rew_mem((uint32_t *)(rccbase+TOFF_RCC_CR)) & RCC_CR_PLL1RDY) == 0)
		return -1;
	sil_modw_mem((uint32_t *)(rccbase+TOFF_RCC_CFGR), RCC_CFGR_SW, REQUEST_SYSCLOCK);

	/*
	 *  VXeNbNݒ蔽f҂
	 */
	tick = 0;
	while(get_sysclk_source() != (REQUEST_SYSCLOCK << CFGR_SWS_SHIFT)){
		if( tick++ > CLOCKSWITCH_TIMEOUT_VALUE){
			return -1;
		}
		sil_dly_nse(1000);
	}

	/*
	 *  HCLKݒ
	 */
	if(REQUEST_HCLK_DIV < (sil_rew_mem((uint32_t *)(rccbase+TOFF_RCC_D1CFGR)) & RCC_D1CFGR_HPRE))
		sil_modw_mem((uint32_t *)(rccbase+TOFF_RCC_D1CFGR), RCC_D1CFGR_HPRE, REQUEST_HCLK_DIV);

	/*
	 *  FLASHANZXEFCgݒ(ݒl葬ꍇ)
	 */
	if(REQUEST_FLASH_LATENCY < (sil_rew_mem((uint32_t *)(TADR_FLASH_R_BASE+TOFF_FLASH_ACR)) & FLASH_ACR_LATENCY)){
		if(Set_Flash_latency(FLASH_ACR_LATENCY) != 0)
			return -3;
	}

	/*
	 *  D1PCLK1ݒ
	 */
	if(REQUEST_APB3_DIV < (sil_rew_mem((uint32_t *)(rccbase+TOFF_RCC_D1CFGR)) & RCC_D1CFGR_D1PPRE))
		sil_modw_mem((uint32_t *)(rccbase+TOFF_RCC_D1CFGR), RCC_D1CFGR_D1PPRE, REQUEST_APB3_DIV);

	/*
	 *  PCLK1ݒ
	 */
	if(REQUEST_APB1_DIV < (sil_rew_mem((uint32_t *)(rccbase+TOFF_RCC_D2CFGR)) & RCC_D2CFGR_D2PPRE1))
		sil_modw_mem((uint32_t *)(rccbase+TOFF_RCC_D2CFGR), RCC_D2CFGR_D2PPRE1, REQUEST_APB1_DIV);

	/*
	 *  PCLK2ݒ
	 */
	if(REQUEST_APB2_DIV < (sil_rew_mem((uint32_t *)(rccbase+TOFF_RCC_D2CFGR)) & RCC_D2CFGR_D2PPRE2))
		sil_modw_mem((uint32_t *)(rccbase+TOFF_RCC_D2CFGR), RCC_D2CFGR_D2PPRE2, REQUEST_APB2_DIV);

	/*
	 *  D3PCLK1ݒ
	 */
	if(REQUEST_APB4_DIV < (sil_rew_mem((uint32_t *)(rccbase+TOFF_RCC_D3CFGR)) & RCC_D3CFGR_D3PPRE))
		sil_modw_mem((uint32_t *)(rccbase+TOFF_RCC_D3CFGR), RCC_D3CFGR_D3PPRE, REQUEST_APB4_DIV);

	/*
	 *  VXeNbNlݒ
	 */
	tmp = sil_rew_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_D1CFGR));
	SystemCoreClock = RCC_GetSysClockFreq() >> ((D1CorePrescTable[(tmp & RCC_D1CFGR_D1CPRE) >> D1CFGR_D1CPRE_SHIFT]) & 0x1F);
	return 0;
}

/*
 *  ݂SYSCLK, HCLK, PCLK1, PCLK2̐ݒlo֐
 */
void
RCC_GetClocksFreq(RCC_ClocksTypeDef* RCC_Clocks)
{
	uint32_t d1cfgr = sil_rew_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_D1CFGR));
	uint32_t d2cfgr = sil_rew_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_D2CFGR));
	uint32_t d3cfgr = sil_rew_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_D3CFGR));

	SystemCoreClock = RCC_GetSysClockFreq() >> (D1CorePrescTable[(d1cfgr & RCC_D1CFGR_D1CPRE)>> D1CFGR_D1CPRE_SHIFT] & 0x1F);
	RCC_Clocks->SYSCLK_Frequency  = SystemCoreClock;
	SystemD2Clock = SystemCoreClock >> ((D1CorePrescTable[(d1cfgr & RCC_D1CFGR_HPRE) >> D1CFGR_HPRE_SHIFT]) & 0x1F);
	RCC_Clocks->HCLK_Frequency    = SystemD2Clock;
	RCC_Clocks->PCLK1_Frequency   = SystemD2Clock >> ((D1CorePrescTable[(d2cfgr & RCC_D2CFGR_D2PPRE1) >> D2CFGR_D2PPRE1_SHIFT]) & 0x1F);
	RCC_Clocks->PCLK2_Frequency   = SystemD2Clock >> ((D1CorePrescTable[(d2cfgr & RCC_D2CFGR_D2PPRE2) >> D2CFGR_D2PPRE2_SHIFT]) & 0x1F);
	RCC_Clocks->D3PCLK1_Frequency = SystemD2Clock >> ((D1CorePrescTable[(d3cfgr & RCC_D3CFGR_D3PPRE) >> D3CFGR_D3PPRE_SHIFT]) & 0x1F);
	RCC_GetPLL2ClockFreq((uint32_t *)&RCC_Clocks->PLL2_Frequency[0]);
	RCC_GetPLL3ClockFreq((uint32_t *)&RCC_Clocks->PLL3_Frequency[0]);
}

