/*
 *  rommon Simple Monitor
 * 
 *  Copyright (C) 1999-2004 by Dep. of Computer Science and Engineering
 *                   Tomakomai National College of Technology, JAPAN
 *  Copyright (C) 2015-2019 by Roi Takeuchi
 *
 *  嵭Ԥϡʲ (1)(4) ξ狼Free Software Foundation 
 *  ˤäƸɽƤ GNU General Public License  Version 2 ˵
 *  ҤƤ˸¤ꡤܥեȥܥեȥ
 *  ѤΤޤࡥʲƱˤѡʣѡۡʰʲ
 *  ѤȸƤ֡ˤ뤳Ȥ̵ǵ롥
 *  (1) ܥեȥ򥽡ɤηѤˤϡ嵭
 *      ɽѾ浪Ӳ̵ݾڵ꤬Τޤޤηǥ
 *      ˴ޤޤƤ뤳ȡ
 *  (2) ܥեȥ򡤥饤֥ʤɡ¾Υեȥȯ˻
 *      ѤǤǺۤˤϡۤȼɥȡ
 *      ԥޥ˥奢ʤɡˤˡ嵭ɽѾ浪Ӳ
 *      ̵ݾڵǺܤ뤳ȡ
 *  (3) ܥեȥ򡤵Ȥ߹ʤɡ¾Υեȥȯ˻
 *      ѤǤʤǺۤˤϡΤ줫ξ
 *      ȡ
 *    (a) ۤȼɥȡѼԥޥ˥奢ʤɡˤˡ嵭
 *        ɽѾ浪Ӳ̵ݾڵǺܤ뤳ȡ
 *    (b) ۤη֤̤ˡˤäơTOPPERSץȤ
 *        𤹤뤳ȡ
 *  (4) ܥեȥѤˤľŪޤϴŪ뤤ʤ»
 *      ⡤嵭ԤTOPPERSץȤդ뤳ȡ
 *
 *  ܥեȥϡ̵ݾڤ󶡤ƤΤǤ롥嵭Ԥ
 *  TOPPERSץȤϡܥեȥ˴ؤơŬѲǽ
 *  ޤơʤݾڤԤʤޤܥեȥѤˤľ
 *  ŪޤϴŪʤ»˴ؤƤ⡤Ǥʤ
 * 
 *  @(#) $Id: $
 */

/*
 *	rommon Serial Functions
 */
#include "sys_defs.h"
#include <string.h>
#include "sil.h"

/*
 *  ץstm32l0xx-nucleo64ѡ
 */

#define sil_modw_mem(addr, mask, val)	sil_wrw_mem((addr), ((sil_rew_mem(addr) & ~(mask)) | (val)))
#define sil_andw_mem(addr, mask)			sil_wrw_mem((addr), (sil_rew_mem(addr) & ~(mask)))
#define sil_orw_mem(addr, val)			sil_wrw_mem((addr), (sil_rew_mem(addr) | (val)))


#define HSI_TIMEOUT_VALUE          (2000U)      /* 2 ms (minimum Tick + 1U) */
#define PLL_TIMEOUT_VALUE          (2000U)      /* 2 ms (minimum Tick + 1U) */
#define MSI_TIMEOUT_VALUE          (2000U)      /* 2 ms (minimum Tick + 1U) */
#define CLOCKSWITCH_TIMEOUT_VALUE  (5000000U)	/* 5 s    */

#if !defined(MSI_RANGE)
#define MSI_RANGE                  0
#endif

#if !defined(HSE_VALUE)
#define HSE_VALUE                  8000000  /* Value of the External oscillator in Hz */
#endif /* HSE_VALUE */
#if !defined(HSI_VALUE)
#define HSI_VALUE                  16000000	/* Value of the Internal oscillator in Hz*/
#endif /* HSI_VALUE */

/*
 *  FLASH_Latency
 */ 
#define FLASH_LATENCY_0            (0x00000000U)    /*!< FLASH Zero Latency cycle */
#define FLASH_LATENCY_1            FLASH_ACR_LATENCY         /*!< FLASH One Latency cycle */

#define PWR_REGULATOR_VOL_SCALE1   PWR_CR_VOS_0
#define PWR_REGULATOR_VOL_SCALE3   PWR_CR_VOS

uint32_t SystemFrequency;

/*
 *  祷ƥ९å
 *  ͤϰʲ̤
 *      System Clock source            = PLL (HSI)
 *      SYSCLK(Hz)                     = 32000000
 *      HCLK(Hz)                       = 32000000
 *      AHB Prescaler                  = 1
 *      APB1 Prescaler                 = 1
 *      APB2 Prescaler                 = 1
 *      HSI Frequency(Hz)              = 16000000
 *      PLL_MUL                        = 4
 *      PLL_DIV                        = 2
 *      Flash Latency(WS)              = 1
 *  return  ｪλʤ0֤
 */
static int
SystemClock_Config(void)
{
	uint32_t tick = 0;

	/* PRFTEN ON */
	sil_orw_mem((uint32_t *)(TADR_FLASH_R_BASE+TOFF_FLASH_ACR), FLASH_ACR_PRFTEN);

	/* Enable Power Control clock */
	sil_orw_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_APB1ENR), RCC_APB1ENR_PWREN);

	/* select the voltage scaling value to 1 */
	sil_modw_mem((uint32_t *)(TADR_PWR_BASE+TOFF_PWR_CR), PWR_CR_VOS, PWR_REGULATOR_VOL_SCALE1);

	/* Disable Power Control clock */
	sil_andw_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_APB1ENR), RCC_APB1ENR_PWREN);

	/* PLLON disable */
	sil_andw_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_CR), RCC_CR_PLLON);

    /* HSI configuration and activation */
	sil_orw_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_CR), RCC_CR_HSION);

	/* HSIDIVEN disable */
	sil_andw_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_CR), RCC_CR_HSIDIVEN);

	/* wait HSI ready */
	tick = 0;
	while((sil_rew_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_CR)) & RCC_CR_HSIRDY) == 0){
		if(tick  > HSI_TIMEOUT_VALUE)
			return -1;
		tick++;
		sil_dly_nse(1000);
    }

	/* PLL */
	sil_andw_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_CR), RCC_CR_PLLON);

	tick = 0;
	while((sil_rew_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_CR)) & RCC_CR_PLLRDY) != 0){
		if(tick  > PLL_TIMEOUT_VALUE)
			return -1;
		tick++;
		sil_dly_nse(1000);
	}

    /* Main PLL configuration and activation */
	sil_modw_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_CFGR),
		(RCC_CFGR_PLLSRC | RCC_CFGR_PLLMUL | RCC_CFGR_PLLDIV), (RCC_CFGR_PLLSRC_HSI | RCC_CFGR_PLLMUL4 | RCC_CFGR_PLLDIV2));

	/* PLL enable */
	sil_orw_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_CR), RCC_CR_PLLON);
	tick = 0;
	while((sil_rew_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_CR)) & RCC_CR_PLLRDY) == 0){
		if(tick  > PLL_TIMEOUT_VALUE)
			return -1;
		tick++;
		sil_dly_nse(1000);
	}

    /* Set new latency */
	sil_modw_mem((uint32_t *)(TADR_FLASH_R_BASE+TOFF_FLASH_ACR), FLASH_ACR_LATENCY, FLASH_LATENCY_1);

    /* Sysclk activation on the main PLL */
	sil_modw_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_CFGR), RCC_CFGR_HPRE, RCC_CFGR_HPRE_DIV1);

	/* select PLL to SYSCLOCK */
	sil_modw_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_CFGR), RCC_CFGR_SW, RCC_CFGR_SW_PLL);
	tick = 0;
	while((sil_rew_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_CFGR)) & RCC_CFGR_SWS) != RCC_CFGR_SWS_PLL){
		if(tick > CLOCKSWITCH_TIMEOUT_VALUE)
			return -1;
		tick++;
		sil_dly_nse(1000);
    };

    /* Set APB1 & APB2 prescaler*/
	sil_modw_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_CFGR), RCC_CFGR_PPRE1, RCC_CFGR_PPRE1_DIV1);
	sil_modw_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_CFGR), RCC_CFGR_PPRE2, RCC_CFGR_PPRE2_DIV1);

	/* MSI disable */
	sil_andw_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_CR), RCC_CR_MSION);
	tick = 0;
	while((sil_rew_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_CR)) & RCC_CR_MSIRDY) != 0){
		if(tick  > MSI_TIMEOUT_VALUE)
			return -1;
		tick++;
		sil_dly_nse(1000);
	}
	SystemFrequency = 32000000;
	return 0;
}

/*
 *  MSIˤ륯å
 *  ͤϰʲ̤
 *      System Clock source            = MSI
 *      SYSCLK(Hz)                     = 1<<(MSI_RANGE+1) * 32768
 *      HCLK(Hz)                       = 1<<(MSI_RANGE+1) * 32768
 *      AHB Prescaler                  = 1
 *      APB1 Prescaler                 = 1
 *      APB2 Prescaler                 = 1
 *      Flash Latency(WS)              = 0
 *      Main regulator output voltage  = Scale3 mode
 *  param   msirange
 *  return  ｪλʤ0֤
 */
static int
SystemLowClock_Config(uint8_t msirange)
{
	uint32_t tick = 0;

	/* MSI enable */
	sil_orw_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_CR), RCC_CR_MSION);

	/* wait MSI ready */
	tick = 0;
	while((sil_rew_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_CR)) & RCC_CR_MSIRDY) == 0){
		if(tick  > MSI_TIMEOUT_VALUE)
			return -1;
		tick++;
		sil_dly_nse(1000);
    }

	/* MSI clock range */
	sil_modw_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_ICSCR), RCC_ICSCR_MSIRANGE, msirange << 13);

	/* Adjusts the Multiple Speed oscillator (MSI) calibration value.*/
	sil_modw_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_ICSCR), RCC_ICSCR_MSITRIM, (0x00 << 24));

	/* PLL disable */
	sil_andw_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_CR), RCC_CR_PLLON);

    /* Sysclk activation on the main PLL */
	sil_modw_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_CFGR), RCC_CFGR_HPRE, RCC_CFGR_HPRE_DIV1);

	/* select MSI to SYSCLOCK */
	sil_modw_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_CFGR), RCC_CFGR_SW, RCC_CFGR_SW_MSI);
	tick = 0;
	while((sil_rew_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_CFGR)) & RCC_CFGR_SWS) != RCC_CFGR_SWS_MSI){
		if(tick > CLOCKSWITCH_TIMEOUT_VALUE)
			return -1;
		tick++;
		sil_dly_nse(1000);
    };

	/* Program the new number of wait states to the LATENCY bits in the FLASH_ACR register */
	sil_modw_mem((uint32_t *)(TADR_FLASH_R_BASE+TOFF_FLASH_ACR), FLASH_ACR_LATENCY, FLASH_LATENCY_0);

	/* PRFTEN ON */
	sil_orw_mem((uint32_t *)(TADR_FLASH_R_BASE+TOFF_FLASH_ACR), FLASH_ACR_PRFTEN);

	/* Enable Power Control clock */
	sil_orw_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_APB1ENR), RCC_APB1ENR_PWREN);

	/* select the voltage scaling value to 3 */
	sil_modw_mem((uint32_t *)(TADR_PWR_BASE+TOFF_PWR_CR), PWR_CR_VOS, PWR_REGULATOR_VOL_SCALE3);

	/* Disable Power Control clock */
	sil_andw_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_APB1ENR), RCC_APB1ENR_PWREN);

    /* Set APB1 & APB2 prescaler*/
	sil_modw_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_CFGR), RCC_CFGR_PPRE1, RCC_CFGR_PPRE1_DIV1);
	sil_modw_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_CFGR), RCC_CFGR_PPRE2, RCC_CFGR_PPRE2_DIV1);
	SystemFrequency = (32768 * (1<<(msirange+1)));
	return 0;
}

/*
 *  ưåμФ
 */
uint32_t
sysclock_init_value(void)
{
#if MSI_RANGE != 0
	return (32768 * (1<<(MSI_RANGE+1)));
#else
	return 32000000;	/* 32 MHz */
#endif
}

/*
 *  ƥ९åѹ
 */
int
sysclock_change(uint8_t range)
{
	if(range == 0)
		return SystemClock_Config();
	else
		return SystemLowClock_Config(range);
}

/*
 *  ٥ΥåȰ¸ν
 *
 *  ȥåץ⥸塼ǡν˸ƤӽФ롥
 */
void
SystemInit(void)
{
	sil_wrw_mem((uint32_t *)(TADR_SCB_BASE+TOFF_SCB_VTOR), (FLASH_BASE));

	/* Set MSION bit */
	sil_orw_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_CR), RCC_CR_MSION);

	/* Reset SW[1:0], HPRE[3:0], PPRE1[2:0], PPRE2[2:0], MCOSEL[2:0] and MCOPRE[2:0] bits */
	sil_andw_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_CFGR),
			(RCC_CFGR_SW | RCC_CFGR_HPRE | RCC_CFGR_PPRE1 | RCC_CFGR_PPRE2 | RCC_CFGR_MCOSEL |RCC_CFGR_MCOPRE));

	/* Reset HSION, HSIDIVEN, HSEON, CSSON and PLLON bits */
	sil_andw_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_CR),
			(RCC_CR_HSION | RCC_CR_HSIDIVEN |RCC_CR_HSEON | RCC_CR_CSSHSEON | RCC_CR_PLLON));

	/* Reset HSI48ON  bit */
	sil_andw_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_CRRCR), RCC_CRRCR_HSI48ON);

	/* Reset HSEBYP bit */
	sil_andw_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_CR), RCC_CR_HSEBYP);

	/* Reset PLLSRC, PLLMUL[3:0] and PLLDIV[1:0] bits */
	sil_andw_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_CFGR), (RCC_CFGR_PLLSRC | RCC_CFGR_PLLMUL | RCC_CFGR_PLLDIV));

	/* Disable all interrupts */
	sil_wrw_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_CIER), 0x00000000);

	if(sysclock_change(MSI_RANGE) != 0){
		while(1){}
	}

	/*
	 *  PCLK
	 */
	sil_wrw_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_CCIPR), 0x00000000);
}

/*
 *  CortexM0+LGϥ٥ѹǽ
 */
void SetVector(void *addr)
{
	sil_wrw_mem((uint32_t *)(TADR_SCB_BASE+TOFF_SCB_VTOR), (uint32_t)addr);
}

#define RCC_ICSCR_MSIRANGE_BITNUMBER 13
#define RCC_CFGR_PLLMUL_BITNUMBER    18
#define RCC_CFGR_PLLDIV_BITNUMBER    22


static const uint8_t APBPrescTable[8] = {0U, 0U, 0U, 0U, 1U, 2U, 3U, 4U};
static const uint8_t PLLMulTable[9] = {3U, 4U, 6U, 8U, 12U, 16U, 24U, 32U, 48U};

/*
 *  ߤSYSCLK, HCLK, PCLK1, PCLK2ͤФؿ
 */
void
RCC_GetClocksFreq(RCC_ClocksTypeDef* RCC_Clocks)
{
	uint32_t pllm = 0, plld = 0, pllvco = 0, msiclkrange = 0;
	uint32_t sysclockfreq = 0;
	uint32_t presc, tmp;
	tmp = sil_rew_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_CFGR));

	/* Get SYSCLK source -------------------------------------------------------*/
	switch (tmp & RCC_CFGR_SWS){
	case RCC_CFGR_SWS_HSI:	/* HSI used as system clock source */
		if((sil_rew_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_CR)) & RCC_CR_HSIDIVF) != 0)
			sysclockfreq =  (HSI_VALUE >> 2);
		else
			sysclockfreq =  HSI_VALUE;
		break;
	case RCC_CFGR_SWS_HSE:	/* HSE used as system clock */
		sysclockfreq = HSE_VALUE;
		break;
	case RCC_CFGR_SWS_PLL:	/* PLL used as system clock */
		pllm = PLLMulTable[(uint32_t)(tmp & RCC_CFGR_PLLMUL) >> RCC_CFGR_PLLMUL_BITNUMBER];
		plld = ((uint32_t)(tmp & RCC_CFGR_PLLDIV) >> RCC_CFGR_PLLDIV_BITNUMBER) + 1;
		tmp  = sil_rew_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_CFGR));
		if((tmp & RCC_CFGR_PLLSRC) != RCC_CFGR_PLLSRC_HSI){
			/* HSE used as PLL clock source */
			pllvco = (HSE_VALUE * pllm) / plld;
		}
		else{
			if((sil_rew_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_CR)) & RCC_CR_HSIDIVF) != 0){
				pllvco = ((HSI_VALUE >> 2) * pllm) / plld;
			}
			else{
				pllvco = (HSI_VALUE * pllm) / plld;
			}
		}
		sysclockfreq = pllvco;
		break;
	case RCC_CFGR_SWS_MSI:	/* MSI used as system clock source */
	default: /* MSI used as system clock */
		msiclkrange = (sil_rew_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_ICSCR)) & RCC_ICSCR_MSIRANGE ) >> RCC_ICSCR_MSIRANGE_BITNUMBER;
		sysclockfreq = (32768 * (1 << (msiclkrange + 1)));
		break;
	}
	RCC_Clocks->SYSCLK_Frequency = sysclockfreq;

	/* HCLK clock frequency */
	RCC_Clocks->HCLK_Frequency = sysclockfreq;

	/* Get PCLK1 prescaler */
	tmp = sil_rew_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_CFGR)) & RCC_CFGR_PPRE1;
	tmp = tmp >> 8;
	presc = APBPrescTable[tmp];
	/* PCLK1 clock frequency */
	RCC_Clocks->PCLK1_Frequency = RCC_Clocks->HCLK_Frequency >> presc;

	/* Get PCLK2 prescaler */
	tmp = sil_rew_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_CFGR)) & RCC_CFGR_PPRE2;
	tmp = tmp >> 11;
	presc = APBPrescTable[tmp];
	/* PCLK2 clock frequency */
	RCC_Clocks->PCLK2_Frequency = RCC_Clocks->HCLK_Frequency >> presc;
}

