/*
 *  rommon Simple Monitor
 * 
 *  Copyright (C) 1999-2004 by Dep. of Computer Science and Engineering
 *                   Tomakomai National College of Technology, JAPAN
 *  Copyright (C) 2015-2018 by Roi Takeuchi
 *
 *  嵭Ԥϡʲ (1)(4) ξ狼Free Software Foundation 
 *  ˤäƸɽƤ GNU General Public License  Version 2 ˵
 *  ҤƤ˸¤ꡤܥեȥܥեȥ
 *  ѤΤޤࡥʲƱˤѡʣѡۡʰʲ
 *  ѤȸƤ֡ˤ뤳Ȥ̵ǵ롥
 *  (1) ܥեȥ򥽡ɤηѤˤϡ嵭
 *      ɽѾ浪Ӳ̵ݾڵ꤬Τޤޤηǥ
 *      ˴ޤޤƤ뤳ȡ
 *  (2) ܥեȥ򡤥饤֥ʤɡ¾Υեȥȯ˻
 *      ѤǤǺۤˤϡۤȼɥȡ
 *      ԥޥ˥奢ʤɡˤˡ嵭ɽѾ浪Ӳ
 *      ̵ݾڵǺܤ뤳ȡ
 *  (3) ܥեȥ򡤵Ȥ߹ʤɡ¾Υեȥȯ˻
 *      ѤǤʤǺۤˤϡΤ줫ξ
 *      ȡ
 *    (a) ۤȼɥȡѼԥޥ˥奢ʤɡˤˡ嵭
 *        ɽѾ浪Ӳ̵ݾڵǺܤ뤳ȡ
 *    (b) ۤη֤̤ˡˤäơTOPPERSץȤ
 *        𤹤뤳ȡ
 *  (4) ܥեȥѤˤľŪޤϴŪ뤤ʤ»
 *      ⡤嵭ԤTOPPERSץȤդ뤳ȡ
 *
 *  ܥեȥϡ̵ݾڤ󶡤ƤΤǤ롥嵭Ԥ
 *  TOPPERSץȤϡܥեȥ˴ؤơŬѲǽ
 *  ޤơʤݾڤԤʤޤܥեȥѤˤľ
 *  ŪޤϴŪʤ»˴ؤƤ⡤Ǥʤ
 * 
 *  @(#) $Id: $
 */
/*******************************************************************************
  * @attention
  *
  * <h2><center>&copy; COPYRIGHT(c) 2017 STMicroelectronics</center></h2>
  *
  * Redistribution and use in source and binary forms, with or without modification,
  * are permitted provided that the following conditions are met:
  *   1. Redistributions of source code must retain the above copyright notice,
  *      this list of conditions and the following disclaimer.
  *   2. Redistributions in binary form must reproduce the above copyright notice,
  *      this list of conditions and the following disclaimer in the documentation
  *      and/or other materials provided with the distribution.
  *   3. Neither the name of STMicroelectronics nor the names of its contributors
  *      may be used to endorse or promote products derived from this software
  *      without specific prior written permission.
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
  * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
  * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
  * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
  ******************************************************************************
  */

/*
 *	rommon Serial Functions
 */
#include "sys_defs.h"
#include <string.h>
#include "sil.h"

#define sil_orw_mem(a, b)		sil_wrw_mem((a), sil_rew_mem(a) | (b))
#define sil_andw_mem(a, b)		sil_wrw_mem((a), sil_rew_mem(a) & ~(b))
#define sil_modw_mem(a, b, c)	sil_wrw_mem((a), (sil_rew_mem(a) & (~b)) | (c))

#define VECT_TAB_OFFSET     0x00		/* Vector Table base offset field. 
                                		   This value must be a multiple of 0x200. */
#define MSI_TIMEOUT_VALUE   (2*1000)	/* 2 ms (minimum Tick + 1) */
#define PLL_TIMEOUT_VALUE   (2*1000)	/* 2 ms (minimum Tick + 1) */
#define HSI48_TIMEOUT_VALUE (2*1000)	/* 2 ms (minimum Tick + 1) */
#define LSE_TIMEOUT_VALUE   (5000*1000)
#define DBP_TIMEOUT_VALUE   (100*1000)

#define CLOCKSWITCH_TIMEOUT_VALUE  (5000*1000) /* 5 s    */
#if BOARDNO >= 1
#define RCC_MSIRANGE        RCC_CR_MSIRANGE_11
#else
#define RCC_MSIRANGE        RCC_CR_MSIRANGE_6
#endif

#define HSE_STARTUP_TIMEOUT 0x0500		/* Time out for HSE start up */
#if !defined  (HSE_VALUE)
  #define HSE_VALUE         8000000	/*!< Value of the External oscillator in Hz */
#endif /* HSE_VALUE */

#if !defined  (HSI_VALUE)
  #define HSI_VALUE         16000000	/*!< Value of the Internal oscillator in Hz*/
#endif /* HSI_VALUE */

/* PLL_VCO = (HSE_VALUE or HSI_VALUE / PLL_M) * PLL_N */
#if BOARDNO == 2 && !defined(USBCLOCK_HSI48)
#define PLL_M      12
#elif BOARDNO == 1
#define PLL_M      6
#else
#define PLL_M      1
#endif
#if BOARDNO < 2
#define PLL_N      40
#else
#define PLL_N      60
#endif

/* SYSCLK = PLL_VCO / PLL_P */
#define PLL_P      7

/* USB OTG FS, SDIO and RNG Clock =  PLL_VCO / PLLQ */
#if BOARDNO < 2
#define PLL_Q      4
#else
#define PLL_Q      2
#endif

/* I2SCLK =  PLLVCO / PLLR */
#if BOARDNO == 1
#define PLL_R      4
#else
#define PLL_R      2	/* Possible value between 2 and 7 */
#endif


#define RCC_PLL_SYSCLK            RCC_PLLCFGR_PLLREN	/* PLLCLK selection from main PLL */

#define RCC_MSICALIBRATION_DEFAULT 0	/* Default MSI calibration trimming value */

/*
 *  RCC_MSI_Clock_Range MSI Clock Range
 */
#define RCC_MSIRANGE_0            RCC_CR_MSIRANGE_0  /* MSI = 100 KHz  */
#define RCC_MSIRANGE_1            RCC_CR_MSIRANGE_1  /* MSI = 200 KHz  */
#define RCC_MSIRANGE_2            RCC_CR_MSIRANGE_2  /* MSI = 400 KHz  */
#define RCC_MSIRANGE_3            RCC_CR_MSIRANGE_3  /* MSI = 800 KHz  */
#define RCC_MSIRANGE_4            RCC_CR_MSIRANGE_4  /* MSI = 1 MHz    */
#define RCC_MSIRANGE_5            RCC_CR_MSIRANGE_5  /* MSI = 2 MHz    */
#define RCC_MSIRANGE_6            RCC_CR_MSIRANGE_6  /* MSI = 4 MHz    */
#define RCC_MSIRANGE_7            RCC_CR_MSIRANGE_7  /* MSI = 8 MHz    */
#define RCC_MSIRANGE_8            RCC_CR_MSIRANGE_8  /* MSI = 16 MHz   */
#define RCC_MSIRANGE_9            RCC_CR_MSIRANGE_9  /* MSI = 24 MHz   */
#define RCC_MSIRANGE_10           RCC_CR_MSIRANGE_10 /* MSI = 32 MHz   */
#define RCC_MSIRANGE_11           RCC_CR_MSIRANGE_11 /* MSI = 48 MHz   */

/*
 * FLASH_Latency FLASH Latency
 */ 
#define FLASH_LATENCY_0           FLASH_ACR_LATENCY_0WS		/* FLASH Zero wait state */
#define FLASH_LATENCY_1           FLASH_ACR_LATENCY_1WS		/* FLASH One wait state */
#define FLASH_LATENCY_2           FLASH_ACR_LATENCY_2WS		/* FLASH Two wait states */
#define FLASH_LATENCY_3           FLASH_ACR_LATENCY_3WS		/* FLASH Three wait states */
#define FLASH_LATENCY_4           FLASH_ACR_LATENCY_4WS		/* FLASH Four wait states */
#if BOARDNO > 1	/* STM32L4RX */
#define FLASH_LATENCY_5           FLASH_ACR_LATENCY_5WS		/* FLASH Five wait state */
#endif

#if BOARDNO < 2
#define TARGET_FLASH_LATENCY      FLASH_LATENCY_4
#define TARGET_HCLK_DIV           RCC_CFGR_HPRE_DIV1
#else
#define TARGET_FLASH_LATENCY      FLASH_LATENCY_3
#define TARGET_HCLK_DIV           RCC_CFGR_HPRE_DIV2
#define AFTERTARGET_FLASH_LATENCY FLASH_LATENCY_5
#endif

#define PWR_FLAG_SETTING_DELAY_US 50	/* Time out value for REGLPF and VOSF flags setting */
#if defined(PWR_CR5_R1MODE)
#define PWR_REGULATOR_VOLTAGE_SCALE1_BOOST  ((uint32_t)0x00000000)  /*!< Voltage scaling range 1 boost mode  */
#endif
#define PWR_REGULATOR_VOLTAGE_SCALE1        PWR_CR1_VOS_0           /*!< Voltage scaling range 1 normal mode */
#define PWR_REGULATOR_VOLTAGE_SCALE2        PWR_CR1_VOS_1           /*!< Voltage scaling range 2             */

/*
 *  defgroup RCCEx_USB_Clock_Source USB Clock Source
 */
#define RCC_USBCLKSOURCE_HSI48         0x00000000
#define RCC_USBCLKSOURCE_PLLSAI1       RCC_CCIPR_CLK48SEL_0
#define RCC_USBCLKSOURCE_PLL           RCC_CCIPR_CLK48SEL_1
#define RCC_USBCLKSOURCE_MSI           RCC_CCIPR_CLK48SEL

#if BOARDNO > 0
#ifdef USBCLOCK_HSI48
#define RCC_USBCLKSOURCE               RCC_USBCLKSOURCE_HSI48
#else
#define RCC_USBCLKSOURCE               RCC_USBCLKSOURCE_MSI
#endif
#endif

/*
 *  defgroup RCC_PLL_Clock_Output PLL Clock Output
 */
#if defined(RCC_PLLSAI2_SUPPORT)
#define RCC_PLL_SAI3CLK                RCC_PLLCFGR_PLLPEN      /*!< PLLSAI3CLK selection from main PLL (for devices with PLLSAI2) */
#else
#define RCC_PLL_SAI2CLK                RCC_PLLCFGR_PLLPEN      /*!< PLLSAI2CLK selection from main PLL (for devices without PLLSAI2) */
#endif /* RCC_PLLSAI2_SUPPORT */
#define RCC_PLL_48M1CLK                RCC_PLLCFGR_PLLQEN      /*!< PLL48M1CLK selection from main PLL */
#define RCC_PLL_SYSCLK                 RCC_PLLCFGR_PLLREN      /*!< PLLCLK selection from main PLL */

/*
 *  defgroup RCCEx_CRS_SynchroSource RCCEx CRS SynchroSource
 */
#define RCC_CRS_SYNC_SOURCE_GPIO       0x00000000U             /*!< Synchro Signal source GPIO */
#define RCC_CRS_SYNC_SOURCE_LSE        CRS_CFGR_SYNCSRC_0      /*!< Synchro Signal source LSE */
#define RCC_CRS_SYNC_SOURCE_USB        CRS_CFGR_SYNCSRC_1      /*!< Synchro Signal source USB SOF (default)*/

/*
 *  defgroup RCCEx_CRS_SynchroDivider RCCEx CRS SynchroDivider
 */
#define RCC_CRS_SYNC_DIV1        0x00000000U                               /* Synchro Signal not divided (default) */
#define RCC_CRS_SYNC_DIV2        CRS_CFGR_SYNCDIV_0                        /* Synchro Signal divided by 2 */
#define RCC_CRS_SYNC_DIV4        CRS_CFGR_SYNCDIV_1                        /* Synchro Signal divided by 4 */
#define RCC_CRS_SYNC_DIV8        (CRS_CFGR_SYNCDIV_1 | CRS_CFGR_SYNCDIV_0) /* Synchro Signal divided by 8 */
#define RCC_CRS_SYNC_DIV16       CRS_CFGR_SYNCDIV_2                        /* Synchro Signal divided by 16 */
#define RCC_CRS_SYNC_DIV32       (CRS_CFGR_SYNCDIV_2 | CRS_CFGR_SYNCDIV_0) /* Synchro Signal divided by 32 */
#define RCC_CRS_SYNC_DIV64       (CRS_CFGR_SYNCDIV_2 | CRS_CFGR_SYNCDIV_1) /* Synchro Signal divided by 64 */
#define RCC_CRS_SYNC_DIV128      CRS_CFGR_SYNCDIV                          /* Synchro Signal divided by 128 */

/*
 *  defgroup RCCEx_CRS_ErrorLimitDefault RCCEx CRS ErrorLimitDefault
 */
#define RCC_CRS_ERRORLIMIT_DEFAULT     0x00000022U   /*!< Default Frequency error limit */


extern int MonVector[];

const unsigned char APBPrescTable[8] =  {0, 0, 0, 0, 1, 2, 3, 4};
const unsigned int  MSIRangeTable[12] = {100000, 200000, 400000, 800000, 1000000, 2000000, \
                                      4000000, 8000000, 16000000, 24000000, 32000000, 48000000};

void RCC_GetClocksFreq(RCC_ClocksTypeDef* RCC_Clocks);


void SetVector(void *addr)
{
	sil_wrw_mem((uint32_t *)(TADR_SCB_BASE+TOFF_SCB_VTOR), (unsigned int)addr | VECT_TAB_OFFSET);	/* Vector Table Relocation */
}

/*
 *  FLASH LATENCY
 *  param  msirange  MSI range value from RCC_MSIRANGE_0 to RCC_MSIRANGE_11
 *  return 0:OK, -1:ERROR
 */
static int
RCC_SetFlashLatencyFromMSIRange(unsigned int msirange)
{
	unsigned int vos = 0;
	unsigned int latency = FLASH_LATENCY_0;  /* default value 0WS */

	if((sil_rew_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_APB1ENR1)) & RCC_APB1ENR1_PWREN) != 0){
		vos = sil_rew_mem((uint32_t *)(TADR_PWR_BASE+TOFF_PWR_CR1)) & PWR_CR1_VOS;
	}
	else{
		sil_orw_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_APB1ENR1), RCC_APB1ENR1_PWREN);
		vos = sil_rew_mem((uint32_t *)(TADR_PWR_BASE+TOFF_PWR_CR1)) & PWR_CR1_VOS;
		sil_andw_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_APB1ENR1), RCC_APB1ENR1_PWREN);
	}

	if(vos == PWR_CR1_VOS_0){
	    if(msirange > RCC_MSIRANGE_8){
			/* MSI > 16Mhz */
			if(msirange > RCC_MSIRANGE_10){
				/* MSI 48Mhz */
				latency = FLASH_LATENCY_2; /* 2WS */
			}
			else{
				/* MSI 24Mhz or 32Mhz */
				latency = FLASH_LATENCY_1; /* 1WS */
			}
		}
		/* else MSI <= 16Mhz default FLASH_LATENCY_0 0WS */
	}
	else{
#if BOARDNO > 1	/* STM32L4RX */
		if(msirange >= RCC_MSIRANGE_8){
			/* MSI >= 16Mhz */
			latency = FLASH_LATENCY_2; /* 2WS */
		}
		else{
			if(msirange == RCC_MSIRANGE_7){
				/* MSI 8Mhz */
				latency = FLASH_LATENCY_1; /* 1WS */
			}
			/* else MSI < 8Mhz default FLASH_LATENCY_0 0WS */
		}
#else	/* STM32L476 */
		if(msirange > RCC_MSIRANGE_8){
			/* MSI > 16Mhz */
			latency = FLASH_LATENCY_3; /* 3WS */
		}
		else{
			if(msirange == RCC_MSIRANGE_8){
				/* MSI 16Mhz */
				latency = FLASH_LATENCY_2; /* 2WS */
			}
			else if(msirange == RCC_MSIRANGE_7) {
				/* MSI 8Mhz */
				latency = FLASH_LATENCY_1; /* 1WS */
			}
			/* else MSI < 8Mhz default FLASH_LATENCY_0 0WS */
		}
#endif
	}
	sil_modw_mem((uint32_t *)(TADR_FLASH_R_BASE+TOFF_FLASH_ACR), FLASH_ACR_LATENCY, latency);

	/*
	 *  FLASH LATENCYǧ
	 */
	if((sil_rew_mem((uint32_t *)(TADR_FLASH_R_BASE+TOFF_FLASH_ACR)) & FLASH_ACR_LATENCY) != latency)
		return -1;
	return 0;
}


/*
 * Configures the System clock source, PLL Multiplier and Divider factors,
 * AHB/APBx prescalers and Flash settings
 * @Note   This function should be called only once the RCC clock configuration
 *         is reset to the default reset state (done in SystemInit() function).
 * @param  None
 * @retval None
 */
static void
SetSysClock(void)
{
	unsigned int msi_range = 0;
	unsigned int tick = 0;
#if BOARDNO > 0
	unsigned int pwrclkchanged = 0;
#endif
#if RCC_USBCLKSOURCE == RCC_USBCLKSOURCE_HSI48
	unsigned int value = 0;
#endif

#if BOARDNO > 1		/* STM32L4RX */
	/* Enable voltage range 1 boost mode for frequency above 80 Mhz */
	sil_orw_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_APB1ENR1), RCC_APB1ENR1_PWREN);

	/* If current range is range 2 */
	if((sil_rew_mem((uint32_t *)(TADR_PWR_BASE+TOFF_PWR_CR1)) & PWR_CR1_VOS) == PWR_REGULATOR_VOLTAGE_SCALE2){
		/* Make sure Range 1 Boost is enabled */
		sil_andw_mem((uint32_t *)(TADR_PWR_BASE+TOFF_PWR_CR5), PWR_CR5_R1MODE);

		/* Set Range 1 */
		sil_modw_mem((uint32_t *)(TADR_PWR_BASE+TOFF_PWR_CR1), PWR_CR1_VOS, PWR_REGULATOR_VOLTAGE_SCALE1);

		/* Wait until VOSF is cleared */
		tick = (PWR_FLAG_SETTING_DELAY_US * (4000000 / 1000000));
		while((sil_rew_mem((uint32_t *)(TADR_PWR_BASE+TOFF_PWR_SR2)) & PWR_SR2_VOSF) != 0){
			if(--tick == 0)
				return;
		}
	}
	/* If current range is range 1 normal or boost mode */
	else{
		/* Enable Range 1 Boost (no issue if bit already reset) */
		sil_andw_mem((uint32_t *)(TADR_PWR_BASE+TOFF_PWR_CR5), PWR_CR5_R1MODE);
	}

	sil_andw_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_APB1ENR1), RCC_APB1ENR1_PWREN);
#endif

#if BOARDNO > 0
	tick = 0;
	if((sil_rew_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_APB1ENR1)) & RCC_APB1ENR1_PWREN) == 0){
		sil_orw_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_APB1ENR1), RCC_APB1ENR1_PWREN);
		pwrclkchanged = 1;
	}

	/*
	 *  LSE ENABLE
	 */
	if((sil_rew_mem((uint32_t *)(TADR_PWR_BASE+TOFF_PWR_CR1)) & PWR_CR1_DBP) == 0){
		sil_orw_mem((uint32_t *)(TADR_PWR_BASE+TOFF_PWR_CR1), PWR_CR1_DBP);
		while((sil_rew_mem((uint32_t *)(TADR_PWR_BASE+TOFF_PWR_CR1)) & PWR_CR1_DBP) == 0){
			if(tick > DBP_TIMEOUT_VALUE)
				return;
			tick++;
			sil_dly_nse(1000);
		}
	}

	sil_orw_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_BDCR), RCC_BDCR_LSEON);
	tick = 0;
	while((sil_rew_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_BDCR)) & RCC_BDCR_LSERDY) == 0){
		if(tick > LSE_TIMEOUT_VALUE)
			return;
		tick++;
		sil_dly_nse(1000);
    }

	if(pwrclkchanged)
		sil_andw_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_APB1ENR1), RCC_APB1ENR1_PWREN);

	/*
	 * Disable the LSE Clock Security System.
	 */
	sil_andw_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_BDCR), RCC_BDCR_LSECSSON);
	sil_andw_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_CIER), RCC_CIFR_LSECSSF);
#endif
#if RCC_USBCLKSOURCE == RCC_USBCLKSOURCE_HSI48 && BOARDNO > 0
      /* Enable the Internal Low Speed oscillator (HSI48). */
	sil_orw_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_CRRCR), RCC_CRRCR_HSI48ON);
	tick = 0;

	/* Wait till HSI48 is ready */
	while((sil_rew_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_CRRCR)) & RCC_CRRCR_HSI48RDY) == 0){
		if(tick > (HSI48_TIMEOUT_VALUE*1000)){
			return;
		}
		tick++;
		sil_dly_nse(1000);
	}
#endif

	/*
	 *  MSI
	 */
    if((sil_rew_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_CFGR)) & RCC_CFGR_SWS) == RCC_CFGR_SWS_MSI){
		/*
		 *  MSI򥷥ƥ९åȤƻѻMSIΥ󥸤ФRCC_MSIRANGEꤷʤ
		 */
		if((sil_rew_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_CR)) & RCC_CR_MSIRGSEL) != 0)
			msi_range = sil_rew_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_CR)) & RCC_CR_MSIRANGE;
		else
			msi_range = (sil_rew_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_CSR)) & RCC_CSR_MSISRANGE) >> 4;
		if(RCC_MSIRANGE > msi_range){
			/*
			 *  ⤤硢FLASH LATENCY
			 */
			if(RCC_SetFlashLatencyFromMSIRange(RCC_MSIRANGE) != 0){
				return;
			}

			/*
			 *  RCC_MSIRANGE˺
			 */
			sil_orw_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_CR), RCC_CR_MSIRGSEL);
			sil_modw_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_CR), RCC_CR_MSIRANGE, RCC_MSIRANGE);
			sil_modw_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_ICSCR), RCC_ICSCR_MSITRIM, RCC_MSICALIBRATION_DEFAULT << 8);
		}
		else{
			/*
			 *  㤤硢RCC_MSIRANGE˺
			 */
			sil_orw_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_CR), RCC_CR_MSIRGSEL);
			sil_modw_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_CR), RCC_CR_MSIRANGE, RCC_MSIRANGE);
			sil_modw_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_ICSCR), RCC_ICSCR_MSITRIM, RCC_MSICALIBRATION_DEFAULT << 8);

			/*
			 *  FLASH LATENCY
			 */
			if(RCC_SetFlashLatencyFromMSIRange(RCC_MSIRANGE) != 0){
				return;
			}
		}
    }
    else{
		/*
		 *  MSISYSCLKǤʤ硢ư
		 */
		sil_orw_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_CR), RCC_CR_MSION);

        /*
		 *  MSI READYԤ
		 */
		tick = 0;
        while((sil_rew_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_CR)) & RCC_CR_MSIRDY) == 0){
			if(tick > MSI_TIMEOUT_VALUE){
				return;
			}
			tick++;
			sil_dly_nse(1000);
		}

		/*
		 *  RCC_MSIRANGE
		 */
		sil_orw_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_CR), RCC_CR_MSIRGSEL);
		sil_modw_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_CR), RCC_CR_MSIRANGE, RCC_MSIRANGE);
		sil_modw_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_ICSCR), RCC_ICSCR_MSITRIM, RCC_MSICALIBRATION_DEFAULT << 8);
	}

#if BOARDNO > 0	/* for USB */
	/*
	 *  Enable MSI Auto-calibration through LSE
	 */
	sil_orw_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_CR), RCC_CR_MSIPLLEN);

	/*
	 *  Select MSI output as USB clock source
	 */
	sil_modw_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_CCIPR), RCC_CCIPR_CLK48SEL, RCC_USBCLKSOURCE_MSI);
#endif

    /*
	 *  PLL
	 */
    if((sil_rew_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_CFGR)) & RCC_CFGR_SWS) != RCC_CFGR_SWS_PLL){
		/*
		 *  PLLSYSCLKǤʤ硢PLL
		 */
		sil_andw_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_CR), RCC_CR_PLLON);

		/*
		 *  PLLԤ
		 */
		tick = 0;
		while((sil_rew_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_CR)) & RCC_CR_PLLRDY) != 0){
			if(tick > PLL_TIMEOUT_VALUE){
				return;
			}
			tick++;
			sil_dly_nse(1000);
 		}

		/*
		 *  PLLå
		 */
		sil_wrw_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_PLLCFGR), ((PLL_M - 1U) << 4) | (PLL_N << 8) | ((PLL_P >> 4) << 17) | \
                   RCC_PLLCFGR_PLLSRC_MSI | (((PLL_Q >> 1) - 1) << 21) | (((PLL_R >> 1) - 1) << 25));

		/*
		 *  PLLƵư
		 */
		sil_orw_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_CR), RCC_CR_PLLON);

		/*
		 *  PLL ƥ९å
		 */
		sil_orw_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_PLLCFGR), RCC_PLL_SYSCLK);

		/*
		 *  PLL READYԤ
		 */
        tick = 0;
		while((sil_rew_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_CR)) & RCC_CR_PLLRDY) == 0){
			if(tick > PLL_TIMEOUT_VALUE){
				return;
			}
			tick++;
			sil_dly_nse(1000);
		}
	}
	else{
		return;
	}

	/*
	 *  ׵FLASH LATENCY⤤硢FLASHLATENCYԤ
	 */
	if(TARGET_FLASH_LATENCY > (sil_rew_mem((uint32_t *)(TADR_FLASH_R_BASE+TOFF_FLASH_ACR)) & FLASH_ACR_LATENCY)){
		/*
		 *  LATENCYꤷǧ
		 */
		sil_modw_mem((uint32_t *)(TADR_FLASH_R_BASE+TOFF_FLASH_ACR), FLASH_ACR_LATENCY, TARGET_FLASH_LATENCY);
		if((sil_rew_mem((uint32_t *)(TADR_FLASH_R_BASE+TOFF_FLASH_ACR)) & FLASH_ACR_LATENCY) != TARGET_FLASH_LATENCY){
			return;
		}
	}

	/*
	 *  SYSCLKPLLѹ塢ǧ
	 */
	sil_modw_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_CFGR), RCC_CFGR_SW, RCC_CFGR_SW_PLL);

	tick = 0;
	while((sil_rew_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_CFGR)) & RCC_CFGR_SWS) != RCC_CFGR_SWS_PLL){
		if(tick > CLOCKSWITCH_TIMEOUT_VALUE){
			return;
		}
		tick++;
		sil_dly_nse(1000);
	}

	/*
	 *  HCLKʬ
	 */
	sil_modw_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_CFGR), RCC_CFGR_HPRE, TARGET_HCLK_DIV);

	/*
	 *  ׵FLASH LATENCY㤤硢FLASHLATENCYԤ
	 */
	if(TARGET_FLASH_LATENCY < (sil_rew_mem((uint32_t *)(TADR_FLASH_R_BASE+TOFF_FLASH_ACR)) & FLASH_ACR_LATENCY)){
		/*
		 *  LATENCYꤷǧ
		 */
		sil_modw_mem((uint32_t *)(TADR_FLASH_R_BASE+TOFF_FLASH_ACR), FLASH_ACR_LATENCY, TARGET_FLASH_LATENCY);
		if((sil_rew_mem((uint32_t *)(TADR_FLASH_R_BASE+TOFF_FLASH_ACR)) & FLASH_ACR_LATENCY) != TARGET_FLASH_LATENCY){
			return;
		}
	}

	/*
	 *  PCLK1ʬͤ
	 */
	sil_modw_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_CFGR), RCC_CFGR_PPRE1, RCC_CFGR_PPRE1_DIV1);

	/*
	 *  PCLK2ʬͤ
	 */
	sil_modw_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_CFGR), RCC_CFGR_PPRE2, RCC_CFGR_PPRE2_DIV1);

#ifdef AFTERTARGET_FLASH_LATENCY
	/*
	 *  ׵FLASH LATENCY⤤硢FLASHLATENCYԤ
	 */
	if(AFTERTARGET_FLASH_LATENCY > (sil_rew_mem((uint32_t *)(TADR_FLASH_R_BASE+TOFF_FLASH_ACR)) & FLASH_ACR_LATENCY)){
		/*
		 *  LATENCYꤷǧ
		 */
		sil_modw_mem((uint32_t *)(TADR_FLASH_R_BASE+TOFF_FLASH_ACR), FLASH_ACR_LATENCY, AFTERTARGET_FLASH_LATENCY);
		if((sil_rew_mem((uint32_t *)(TADR_FLASH_R_BASE+TOFF_FLASH_ACR)) & FLASH_ACR_LATENCY) != AFTERTARGET_FLASH_LATENCY){
			return;
		}
	}

	/*
	 *  HCLKʬ
	 */
	sil_modw_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_CFGR), RCC_CFGR_HPRE, RCC_CFGR_HPRE_DIV1);

	/*
	 *  ׵FLASH LATENCY㤤硢FLASHLATENCYԤ
	 */
	if(AFTERTARGET_FLASH_LATENCY < (sil_rew_mem((uint32_t *)(TADR_FLASH_R_BASE+TOFF_FLASH_ACR)) & FLASH_ACR_LATENCY)){
		/*
		 *  LATENCYꤷǧ
		 */
		sil_modw_mem((uint32_t *)(TADR_FLASH_R_BASE+TOFF_FLASH_ACR), FLASH_ACR_LATENCY, AFTERTARGET_FLASH_LATENCY);
		if((sil_rew_mem((uint32_t *)(TADR_FLASH_R_BASE+TOFF_FLASH_ACR)) & FLASH_ACR_LATENCY) != AFTERTARGET_FLASH_LATENCY){
			return;
		}
	}
#endif	/* AFTERTARGET_FLASH_LATENCY */
#if RCC_USBCLKSOURCE == RCC_USBCLKSOURCE_HSI48
	/*
	 *  CRSͭ
	 */
	sil_orw_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_APB1ENR1), RCC_APB1ENR1_CRSEN);

	/*
	 *  CRSꥻåȸ塢ꥻåȲ
	 */
	sil_orw_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_APB1RSTR1), RCC_APB1RSTR1_CRSRST);
	sil_andw_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_APB1RSTR1), RCC_APB1RSTR1_CRSRST);

	/* Set the SYNCDIV[2:0] bits according to Prescaler value */
	/* Set the SYNCSRC[1:0] bits according to Source value */
	/* Set the SYNCSPOL bit according to Polarity value */
	value = (RCC_CRS_SYNC_DIV1 | RCC_CRS_SYNC_SOURCE_USB | /*pInit->Polarity*/ 0);
	/* Set the RELOAD[15:0] bits according to ReloadValue value */
	value |= ((48000000 / 1000) - 1U);

	/* Set the FELIM[7:0] bits according to ErrorLimitValue value */
	value |= (RCC_CRS_ERRORLIMIT_DEFAULT << /*CRS_CFGR_FELIM_Pos*/16);
	sil_wrw_mem((uint32_t *)(TADR_CRS_BASE+TOFF_CRS_CFGR), value);

	/* Adjust HSI48 oscillator smooth trimming */
	/* Set the TRIM[5:0] bits according to RCC_CRS_HSI48CalibrationValue value */
	sil_modw_mem((uint32_t *)(TADR_CRS_BASE+TOFF_CRS_CR), CRS_CR_TRIM, (/*pInit->HSI48CalibrationValue*/0x20 << /*CRS_CR_TRIM_Pos*/8));

	/* START AUTOMATIC SYNCHRONIZATION*/

	/* Enable Automatic trimming & Frequency error counter */
	sil_orw_mem((uint32_t *)(TADR_CRS_BASE+TOFF_CRS_CR), (CRS_CR_AUTOTRIMEN | CRS_CR_CEN));
#endif
}

/*
 * Setup the microcontroller system
 * Initialize the Embedded Flash Interface, the PLL and update the SystemFrequency variable.
 * @param  None
 * @retval None
 */
void
SystemInit(void)
{
	/* FPU settings ------------------------------------------------------------*/
	sil_orw_mem((uint32_t *)(TADR_SCB_BASE+TOFF_SCB_CPACR), ((3UL << 10*2)|(3UL << 11*2)));	/* set CP10 and CP11 Full Access */

	/* Reset the RCC clock configuration to the default reset state ------------*/
	/* Set MSION bit */
	sil_orw_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_CR), RCC_CR_MSION);

	/* Reset CFGR register */
	sil_wrw_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_CFGR), 0x00000000);

	/* Reset HSEON, CSSON and PLLON bits */
	sil_andw_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_CR), (RCC_CR_PLLSAI2ON | RCC_CR_PLLSAI1ON | RCC_CR_PLLON | RCC_CR_CSSON | RCC_CR_HSEON));

	/* Reset PLLCFGR register */
	sil_wrw_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_PLLCFGR), RCC_PLLCFGR_PLLN_4);

	/* Reset HSEBYP bit */
	sil_andw_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_CR), RCC_CR_HSEBYP);

	/* Disable all interrupts */
	sil_wrw_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_CIER), 0x00000000);

	/* Configure the System clock source, PLL Multiplier and Divider factors, 
       AHB/APBx prescalers and Flash settings ----------------------------------*/
	SetSysClock();

#if BOARDNO > 1
	sil_orw_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_APB1ENR1), RCC_APB1ENR1_PWREN);
	sil_orw_mem((uint32_t *)(TADR_PWR_BASE+TOFF_PWR_CR2), PWR_CR2_IOSV);	/* for LPUART */
	sil_orw_mem((uint32_t *)(TADR_PWR_BASE+TOFF_PWR_CR2), PWR_CR2_USV);		/* for USB */
#endif

	/* Configure the Vector Table location add offset address ------------------*/
	SetVector((void*)MonVector);
}

/*
 * Returns the frequencies of different on chip clocks; SYSCLK, HCLK, 
 * PCLK1 and PCLK2.
 * @note   The system frequency computed by this function is not the real 
 *         frequency in the chip. It is calculated based on the predefined 
 *         constant and the selected clock source:
 * @note     If SYSCLK source is HSI, function returns values based on HSI_VALUE(*)
 * @note     If SYSCLK source is HSE, function returns values based on HSE_VALUE(**)
 * @note     If SYSCLK source is PLL, function returns values based on HSE_VALUE(**) 
 *           or HSI_VALUE(*) multiplied/divided by the PLL factors.         
 * @note     (*) HSI_VALUE is a constant defined in stm32f4xx.h file (default value
 *               16 MHz) but the real value may vary depending on the variations
 *               in voltage and temperature.
 * @note     (**) HSE_VALUE is a constant defined in stm32f4xx.h file (default value
 *                25 MHz), user has to ensure that HSE_VALUE is same as the real
 *                frequency of the crystal used. Otherwise, this function may
 *                have wrong result.
 *
 * @note   The result of this function could be not correct when using fractional
 *         value for HSE crystal.
 *
 * @param  RCC_Clocks: pointer to a RCC_ClocksTypeDef structure which will hold
 *         the clocks frequencies.
 *
 * @note   This function can be used by the user application to compute the 
 *         baudrate for the communication peripherals or configure other parameters.
 * @note   Each time SYSCLK, HCLK, PCLK1 and/or PCLK2 clock changes, this function
 *         must be called to update the structure's field. Otherwise, any
 *         configuration based on this function will be incorrect.
 *
 * @retval None
 */
void
RCC_GetClocksFreq(RCC_ClocksTypeDef* RCC_Clocks)
{
	unsigned int msirange = 0U, pllvco = 0U, pllsource = 0U, pllr = 2U, pllm = 2U;
	unsigned int sysclockfreq = 0;
	unsigned int presc, tmp;

	if(((sil_rew_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_CFGR)) & RCC_CFGR_SWS) == RCC_CFGR_SWS_MSI) ||
     (((sil_rew_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_CFGR)) & RCC_CFGR_SWS) == RCC_CFGR_SWS_PLL) && ((sil_rew_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_PLLCFGR)) & RCC_PLLCFGR_PLLSRC) == RCC_PLLCFGR_PLLSRC_MSI))){
		/* MSI or PLL with MSI source used as system clock source */

		/* Get SYSCLK source */
		if((sil_rew_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_CR)) & RCC_CR_MSIRGSEL) != 0)
			msirange = (sil_rew_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_CR)) & RCC_CR_MSIRANGE) >> 4;
		else
			msirange = (sil_rew_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_CSR)) & RCC_CSR_MSISRANGE) >> 8;
		/*MSI frequency range in HZ*/
		msirange = MSIRangeTable[msirange];

		if((sil_rew_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_CFGR)) & RCC_CFGR_SWS) == RCC_CFGR_SWS_MSI){
			/* MSI used as system clock source */
			sysclockfreq = msirange;
		}
	}
	else if((sil_rew_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_CFGR)) & RCC_CFGR_SWS) == RCC_CFGR_SWS_HSI){
		/* HSI used as system clock source */
		sysclockfreq = HSI_VALUE;
	}
	else if((sil_rew_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_CFGR)) & RCC_CFGR_SWS) == RCC_CFGR_SWS_HSE){
		/* HSE used as system clock source */
		sysclockfreq = HSE_VALUE;
	}

	if((sil_rew_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_CFGR)) & RCC_CFGR_SWS) == RCC_CFGR_SWS_PLL){
		/* PLL used as system clock  source */

		/* PLL_VCO = (HSE_VALUE or HSI_VALUE or MSI_VALUE/ PLLM) * PLLN
		SYSCLK = PLL_VCO / PLLR
		*/
		pllsource = sil_rew_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_PLLCFGR)) & RCC_PLLCFGR_PLLSRC;
		pllm = ((sil_rew_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_PLLCFGR)) & RCC_PLLCFGR_PLLM) >> 4) + 1U ;

		switch (pllsource){
		case RCC_PLLCFGR_PLLSRC_HSI:  /* HSI used as PLL clock source */
			pllvco = (HSI_VALUE / pllm) * ((sil_rew_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_PLLCFGR)) & RCC_PLLCFGR_PLLN) >> 8);
			break;

		case RCC_PLLCFGR_PLLSRC_HSE:  /* HSE used as PLL clock source */
			pllvco = (HSE_VALUE / pllm) * ((sil_rew_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_PLLCFGR)) & RCC_PLLCFGR_PLLN) >> 8);
			break;

		case RCC_PLLCFGR_PLLSRC_MSI:  /* MSI used as PLL clock source */
		default:
			pllvco = (msirange / pllm) * ((sil_rew_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_PLLCFGR)) & RCC_PLLCFGR_PLLN) >> 8);
			break;
		}
		pllr = (((sil_rew_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_PLLCFGR)) & RCC_PLLCFGR_PLLR) >> 25) + 1) * 2;
		sysclockfreq = pllvco/pllr;
	}
	RCC_Clocks->SYSCLK_Frequency = sysclockfreq;

	/* HCLK clock frequency */
	RCC_Clocks->HCLK_Frequency = sysclockfreq;

	/* Get PCLK1 prescaler */
	tmp = sil_rew_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_CFGR)) & RCC_CFGR_PPRE1;
	tmp = tmp >> 8;
	presc = APBPrescTable[tmp];
	/* PCLK1 clock frequency */
	RCC_Clocks->PCLK1_Frequency = RCC_Clocks->HCLK_Frequency >> presc;

	/* Get PCLK2 prescaler */
	tmp = sil_rew_mem((uint32_t *)(TADR_RCC_BASE+TOFF_RCC_CFGR)) & RCC_CFGR_PPRE2;
	tmp = tmp >> 11;
	presc = APBPrescTable[tmp];
	/* PCLK2 clock frequency */
	RCC_Clocks->PCLK2_Frequency = RCC_Clocks->HCLK_Frequency >> presc;
}

